
//sys_clk max 80MHz
//GPIO
//UART0
//Timer0/1
//WatchDog
//SPI Master
//I2C Master
//APB2 Master [1]
//AHB2 Master
`resetall

module Gowin_EMPU_Template (
  sys_clk,
  reset_n,
  gpio,
  test_vdd,
  int_o_test,
//uart interface
  uart0_rxd,
  uart0_txd,
//SPI interface
  miso,
  mosi,
  sclk,
  nss,
//I3C interface
  SCL_I3C,
  SDA_I3C,
//I2C interface
  scl,
  sda
);

input sys_clk;
input reset_n;
inout [15:0] gpio;
input uart0_rxd;
output uart0_txd;
input miso;
output mosi;
output sclk;
output nss;
inout scl;
inout sda;

output int_o_test;
output test_vdd;
inout SCL_I3C;
inout SDA_I3C;

assign test_vdd=1'b1;

/*------------------------- APB2 Master [1] --------------------------*/
wire        master_pclk;
wire        master_rst;
wire        master_penable;
wire [7:0]  master_paddr;
wire        master_pwrite;
wire [31:0] master_pwdata;
wire [3:0]  master_pstrb;
wire        master_psel1;
wire [31:0] master_prdata1;
wire        master_pslverr1 = 1'b0;

//-------------------------------------------//
//---- CLK  define
//-------------------------------------------//
wire i3c_clk_100M,fclk_40M;
wire mcu_clk;

assign mcu_clk =fclk_40M;

// i3c/fclk  = 2.5
    Gowin_PLLVR_i3c pll_100M(
        .clkout(i3c_clk_100M), //output clkout--100M
        .clkin(sys_clk) //input clkin--50M
    );

    Gowin_PLLVR_fclk pll_40M(
        .clkout(fclk_40M), //output clkout--40M
        .clkin(sys_clk) //input clkin--50M
    );


/*------------------------- APB2 Master [1] --------------------------*/
//-------------------------------------------//
//---- Gowin_EMPU_Top 
//-------------------------------------------//
wire empu_int0 = ~int_o;
Gowin_EMPU_Top u_Gowin_EMPU_Top (
  .sys_clk(mcu_clk),
  .reset_n(reset_n),
  .user_int_0(empu_int0),
  //GPIO
  .gpio(gpio),
  //UART0
  .uart0_rxd(uart0_rxd),
  .uart0_txd(uart0_txd),
  //SPI Master
  .miso(miso),
  .mosi(mosi),
  .sclk(sclk),
  .nss(nss),
  //I2C Master
  .scl(scl),
  .sda(sda),
  //----APB2 Master [1]----//
  .master_pclk(master_pclk),
  .master_rst(master_rst),
  .master_penable(master_penable),
  .master_paddr(master_paddr),
  .master_pwrite(master_pwrite),
  .master_pwdata(master_pwdata),
  .master_pstrb(master_pstrb),
  .master_pprot(),
  .master_psel1(master_psel1),
  .master_prdata1(master_prdata1),
  .master_pready1(1'b1),
  .master_pslverr1(master_pslverr1)
);

assign int_o_test =int_o;
wire    i3c_rstn = reset_n;
wire 	[31:0] cpu_addr,cpu_wdat,cpu_rdat;

wire    [7:0]APB_PADDR;
wire    [31:0]APB_PWDATA;
wire    [31:0]APB_PRDATA;
wire    APB_PSEL;
wire    APB_PWRITE;
wire    APB_PENABLE;

  
assign    APB_PADDR         =master_paddr[7:0];
assign    APB_PWDATA        =master_pwdata;
assign    master_prdata1    =APB_PRDATA;
assign    APB_PSEL          =master_psel1;
assign    APB_PWRITE        =master_pwrite;
assign    APB_PENABLE       =master_penable;

I3C_Dual_Clock_Top inst_i3c_dual(
    .SCL                        (SCL_I3C),
    .SDA                        (SDA_I3C),
    .i3c_clk                    (i3c_clk_100M),
    .fclk                       (fclk_40M),
    .rstn                       (i3c_rstn),
    .SDA_PULL                   (),
    .PADDR                      (APB_PADDR[7:0]),
    .PWDATA                     (APB_PWDATA),
    .PRDATA                     (APB_PRDATA),
    .PSEL                       (APB_PSEL),
    .PWRITE                     (APB_PWRITE),
    .PENABLE                    (APB_PENABLE),
    .int_o                      (int_o)
);

endmodule