/*
 * *****************************************************************************************
 *
 * 		Copyright (C) 2014-2019 Gowin Semiconductor Technology Co.,Ltd.
 * 		
 * @file        gw1ns4c_USER0.h
 * @author	    Embedded Development Team
 * @version     V1.0.0
 * @date        2019-10-1 09:00:00
 * @brief       This file contains all the functions prototypes for the USER0 firmware library.
 ******************************************************************************************
 */

/* Define to prevent recursive inclusion -------------------------------------*/
#ifndef __GW1NS4C_USER0_H
#define __GW1NS4C_USER0_H

#ifdef __cplusplus
 extern "C" {
#endif

/* Includes ------------------------------------------------------------------*/
#include "gw1ns4c.h"

	 

/** @addtogroup GW1NS4C_StdPeriph_Driver
  * @{
  */

/** @addtogroup USER0
  * @{
  */

/** @defgroup USER0_Exported_Types
  * @{
  */

/* USER0 interrupt typedef */
typedef FunctionalState USER0Int_TypeDef;  /* ENABLE/DISABLE */

/* USER0 External input typedef */
typedef enum
{
  USER0_DISABLE = 0,  /* External Disable */
  USER0_EXTI_EN,      /* Select external input as enable */
  USER0_EXTI_CLK      /* Select external input as clock */
}USER0Exti_TypeDef;

/** 
  * @brief USER0 Init Structure definition  
  */
typedef struct
{
  uint32_t Reload;                 /* Reload value, sets the current value */

  USER0Int_TypeDef USER0_Int;      /* USER0 interrupt */

  USER0Exti_TypeDef USER0_Exti;    /* Select USER0 external input */

}USER0_InitTypeDef;

/**
  * @}
  */

/** @defgroup USER0_Exported_Macros
  * @{
  */

#define USER0_CTRL_EN_Pos         0  /* USER0 CTRL: Enable Position */
#define USER0_CTRL_SELEXTEN_Pos   1  /* USER0 CTRL: External Enable Position */
#define USER0_CTRL_SELEXTCLK_Pos  2  /* USER0 CTRL: External Clock Position */
#define USER0_CTRL_IRQEN_Pos      3  /* USER0 CTRL: Interrupt Enable Position */

/**
  * @}
  */

/** @defgroup USER0_Exported_Functions
  * @{
  */
typedef struct
{
  __IO   uint32_t  CTRL;          /*!< Offset: 0x000 Control Register (R/W)          */
  __IO   uint32_t  VALUE;         /*!< Offset: 0x004 Current Value Register (R/W)    */
  __IO   uint32_t  RELOAD;        /*!< Offset: 0x008 Reload Value Register  (R/W)    */
  union
  {
    __I    uint32_t  INTSTATUS;   /*!< Offset: 0x00C Interrupt Status Register (R/ ) */
    __O    uint32_t  INTCLEAR;    /*!< Offset: 0x00C Interrupt Clear Register ( /W)  */
  };
}USER0_TypeDef;
/**
  * @brief Initializes USER0 module.
  */
extern void USER0_Init(USER0_TypeDef* USER0x,USER0_InitTypeDef* USER0_InitStruct);

/**
  * @brief Starts USER0.
  */
extern void USER0_StartUSER0(USER0_TypeDef* USER0x);

/**
  * @brief Stops USER0.
  */
extern void USER0_StopUSER0(USER0_TypeDef* USER0x);

/**
  * @brief Returns USER0 IRQ status
  */
extern ITStatus USER0_GetIRQStatus(USER0_TypeDef* USER0x);

/**
  * @brief USER0 interrupt clear
  */
extern void USER0_ClearIRQ(USER0_TypeDef* USER0x);

/**
  * @brief Returns USER0 Reload value.
  */
extern uint32_t USER0_GetReload(USER0_TypeDef* USER0x);

/**
  * @brief Sets USER0 Reload value.
  */
extern void USER0_SetReload(USER0_TypeDef* USER0x,uint32_t value);

/**
  * @brief Returns USER0 current value.
  */
extern uint32_t USER0_GetValue(USER0_TypeDef* USER0x);

/**
  * @brief Sets USER0 current value.
  */
extern void USER0_SetValue(USER0_TypeDef* USER0x,uint32_t value);

/**
  * @brief Enables USER0 Interrupt requests.
  */
extern void USER0_EnableIRQ(USER0_TypeDef* USER0x);

/**
  * @brief Disables USER0 Interrupt requests.
  */
extern void USER0_DisableIRQ(USER0_TypeDef* USER0x);

/**
  * @brief Select Ext input Enable.
  */
extern void USER0_SelExtEnable(USER0_TypeDef *USER0);

/**
  * @brief Select Ext input Clock.
  */
extern void USER0_SelExtClock(USER0_TypeDef *USER0);

/**
  * @}
  */

#ifdef __cplusplus
}
#endif

#endif  /* __GW1NS4C_USER0_H */

/**
  * @}
  */ 

/**
  * @}
  */ 

/************************GowinSemiconductor******END OF FILE*******************/
