#include "gowin_i3c.h"
#include "gw1ns4c.h"


void gowin_i3c_init(Gowin_I3C_TypeDef * I3Cx)
{
	I3Cx->MCFG.MENABLE         = ENABLE;		//1:master; 		0:slave
	I3Cx->MCFG.MDISTIMEOUT     = 0;					//1:disable;		0:enable (Time out)
//	I3Cx->MCFG.PPHIGH          = 8;					//dh=3~15(4 bits), scl_high 	= (dh+1)*fclk
	I3Cx->MCFG.PPHIGH          = 3;					//dh=3~15(4 bits), scl_high 	= (dh+1)*fclk
	I3Cx->MCFG.PPLOWEXTRA      = 0;					//dl=0~15(4 bits), scl_low		=	scl_high	+ dl*fclk =(dh+dl) *fclk
	I3Cx->MCFG.ODSCL           = 50;				//[dol=doh=0~255(8 bits), scl_odh=scl_odl= (dh*(doh+1)) *fclk]
	I3Cx->MCFG.ODHIGHEQUALPP   = 0;					//1:scl_odh =scl_high; 0: scl_odh=scl_odl= (dh*(doh+1)) *fclk
	I3Cx->MCFG.I2CSCL          = 4;					//d2=2~15(4 bits), d2(even):scl_i2h=scl_i2l = (scl_odl *d2) *fclk 
																					//								 d2(odd) :scl_i2l=(scl_odl *d2) *fclk,scl_i2h=(scl_odl *(d2-1)) *fclk,
	
	I3Cx->MIS.SSTART           = ENABLE;
	I3Cx->MIS.MCONTROLFINISH   = DISABLE;
	I3Cx->MIS.COMCOMPLETE      = DISABLE;
	I3Cx->MIS.RFIFONOTEMPTY    = DISABLE;
	I3Cx->MIS.SFIFONOTFULL     = DISABLE;
	I3Cx->MIS.IBIRCV           = ENABLE;
	I3Cx->MIS.ERR              = DISABLE;
		
	return;
}


void gowin_i3c_clear_sfifo(Gowin_I3C_TypeDef * I3Cx)
{
	I3Cx->MDATACONTROL.SFIFOCLR = 1;
}


void gowin_i3c_clear_rfifo(Gowin_I3C_TypeDef * I3Cx)
{
	I3Cx->MDATACONTROL.RFIFOCLR = 1;	
}


uint8_t gowin_i3c_get_sfifo_cnt(Gowin_I3C_TypeDef * I3Cx)
{
	return (I3Cx->MDATACONTROL.SFIFOCNT);
}


uint8_t gowin_i3c_get_rfifo_cnt(Gowin_I3C_TypeDef * I3Cx)
{
	return (I3Cx->MDATACONTROL.RFIFOCNT);
}


void gowin_i3c_set_IBIRspType(Gowin_I3C_TypeDef * I3Cx, uint8_t IBIRspType)
{
	I3Cx->MCONTROL.IBIRSPTYPE = IBIRspType;
}


void I3C_Wait_MControl_Finish(Gowin_I3C_TypeDef *I3Cx)
{
	while(1) {
		if(I3Cx->MSTS.MCONTROLFINISH == 1) {
			break;
		}
	}
	return;
}


void I3C_Wait_Com_Complete(Gowin_I3C_TypeDef *I3Cx)
{
	while(1) {
		if(I3Cx->MSTS.COMCOMPLETE == 1) {
			break;
		}
	}
	return;
}


void I3C_IBI_Manual_Ack(Gowin_I3C_TypeDef *I3Cx, uint8_t ManualAckType)
{
	uint8_t IBIRspTypeBkUp;
	
	IBIRspTypeBkUp            = I3Cx->MCONTROL.IBIRSPTYPE;
	I3Cx->MCONTROL.IBIRSPTYPE = ManualAckType;
	I3Cx->MCONTROL.REQUEST    = I3C_REQUEST_MASTER_IBI_RESP;
	I3C_Wait_MControl_Finish(I3Cx);
	
	I3Cx->MCONTROL.IBIRSPTYPE = IBIRspTypeBkUp;
}


uint8_t I3C_Write(Gowin_I3C_TypeDef *I3Cx, uint8_t ComType, uint8_t SlaveAddr, uint8_t *pTxData, uint32_t DataByteLen)
{
	uint8_t Nack = 0;
	uint32_t i = 0;
	
	// write reg MCONTROL, contains informations about slave addr, com type, direction 	
	I3Cx->MCONTROL.COMTYPE     = ComType;
	I3Cx->MCONTROL.COMADDR     = SlaveAddr;
	I3Cx->MCONTROL.DIRECTION   = I3C_MASTER_DIR_WRITE;
	I3Cx->MCONTROL.READTERMCNT = 0;
	I3Cx->MCONTROL.REQUEST     = I3C_REQUEST_MASTER_STARTnADDR;	
//	printf("  wait flag ......\r\n");
	I3C_Wait_MControl_Finish(I3Cx);
//	printf("  MASTER_STARTnADDR =0x%x  ...\r\n",SlaveAddr);
	
	// if slave nack, return
	if(I3Cx->MSTS.NACK) {
		return 1;
	}
	
	// send tx data to tx fifo, the data will be transmitted
	if(DataByteLen > 0) {
		for(i = 0; i < DataByteLen - 1; i++) {
			I3Cx->MTXB.RegValue = *pTxData;
			pTxData++;
		}
		I3Cx->MTXB.RegValue = (*pTxData) | MTXB_LAST;
		I3C_Wait_Com_Complete(I3Cx);
	}
	Nack = I3Cx->MSTS.NACK;
		
	return Nack;
}


uint8_t I3C_Read(Gowin_I3C_TypeDef *I3Cx, uint8_t ComType, uint8_t SlaveAddr, uint8_t *pRxData, uint32_t DataByteLen)
{
	uint8_t Nack;
	
	// write reg MCONTROL, contains informations about slave addr, com type, direction 
	I3Cx->MCONTROL.COMTYPE     = ComType;
	I3Cx->MCONTROL.COMADDR     = SlaveAddr;
	I3Cx->MCONTROL.DIRECTION   = I3C_MASTER_DIR_READ;
	I3Cx->MCONTROL.READTERMCNT = DataByteLen;
	I3Cx->MCONTROL.REQUEST     = I3C_REQUEST_MASTER_STARTnADDR;
	I3C_Wait_MControl_Finish(I3Cx);
	
	// if slave nack, return
	if(I3Cx->MSTS.NACK) {
		return 1;
	}
	
	// read rx data from rx fifo
	Nack = I3C_ReadRxData(I3Cx, pRxData, DataByteLen);

	return Nack;
}


uint8_t I3C_ReadRxData(Gowin_I3C_TypeDef *I3Cx, uint8_t *pRxData, uint32_t RxDataNum)
{
	uint32_t RxDataCnt = 0;
	if(RxDataNum == 0) {
		return 0;
	}
	
	// read rx data from rx fifo, if nack, return.
	while(1) {
		while(I3Cx->MSTS.RFIFONOTEMPTY == 0) {
			if(I3Cx->MSTS.NACK) {
				return 1;
			}
			else if(I3Cx->MSTS.COMCOMPLETE == 1) {
				return 0;
			}
		}
		*pRxData = I3Cx->MRXB.RDATA;
		pRxData++;
		RxDataCnt++;		
		if(RxDataCnt == RxDataNum) {
			break;
		}
	}

	return 0;
}


void I3C_SendStop(Gowin_I3C_TypeDef *I3Cx)
{
	// write stop request in reg mcontrol request field.
	I3Cx->MCONTROL.REQUEST  = I3C_REQUEST_MASTER_STOP;
	I3C_Wait_MControl_Finish(I3Cx);
}


uint8_t I3C_Write_Slave_Reg(Gowin_I3C_TypeDef *I3Cx, uint8_t ComType, uint8_t SlaveAddr, uint8_t RegAddr, uint8_t TxData)
{
	uint8_t Nack;
	
	
	I3Cx->MTXB.RegValue = RegAddr;
//	printf("  slave RegAddr=0x%x  ...\r\n",RegAddr);
	
	// write reg mcontrol and send data to tx fifo
	Nack = I3C_Write(I3Cx, ComType, SlaveAddr, &TxData, 1);
	
	// stop
	I3C_SendStop(I3Cx);
	return Nack;
}


uint8_t I3C_Read_Slave_Reg(Gowin_I3C_TypeDef *I3Cx, uint8_t ComType, uint8_t SlaveAddr, uint8_t RegAddr, uint8_t *pRxData)
{
	uint8_t Nack;

	// start and send reg addr to slave dynamic addr
	Nack = I3C_Write(I3Cx, ComType, SlaveAddr, &RegAddr, 1);
	if(Nack) {
		I3C_SendStop(I3Cx);
		return Nack;
	}
	
	// re-start and read one byte from slave dynamic addr
	Nack = I3C_Read(I3Cx, ComType, SlaveAddr, pRxData, 1);
	
	// stop
	I3C_SendStop(I3Cx);
	
	return Nack;
}







void I3C_ExitDDR(Gowin_I3C_TypeDef *I3Cx, uint8_t SlaveAddr)
{
	volatile I3C_MCONTROL_RegDef MControlBuf;
	
	// write reg mcontrol, send exit ddr request and slave addr, wait request complete.
	MControlBuf.RegValue      = 0x00000026;
	MControlBuf.COMADDR       = SlaveAddr;
	I3Cx->MCONTROL.RegValue = MControlBuf.RegValue;
	I3C_Wait_MControl_Finish(I3Cx);	
}


uint8_t I3C_DDR_Read(Gowin_I3C_TypeDef *I3Cx, uint8_t SlaveAddr, uint8_t Command, uint8_t *pRxData, uint8_t RxByteNum, uint8_t CRC)
{
	uint8_t Nack;
	
	I3Cx->MCONTROL.COMTYPE       = I3C_COMTYPE_I3C_DDR;
	I3Cx->MCONTROL.COMADDR       = SlaveAddr;
	I3Cx->MCONTROL.DDRENDWITHCRC = CRC;
	I3Cx->MCONTROL.READTERMCNT   = RxByteNum;
	I3Cx->MCONTROL.REQUEST       = 1;
	
	I3Cx->MTXB.RegValue = Command;
	
	I3C_Wait_MControl_Finish(I3Cx);
	
	// read data from rx fifo
	Nack = I3C_ReadRxData(I3C0, pRxData, RxByteNum);

	return Nack;

}


uint8_t I3C_DDR_Write(Gowin_I3C_TypeDef *I3Cx, uint8_t SlaveAddr, uint8_t Command, uint8_t *pTxData, uint8_t TxByteNum, uint8_t CRC)
{
	uint8_t Nack;
	
	// write reg mcontrol, send ddr request, command code and slave addr, wait request complete
	I3Cx->MCONTROL.DDRENDWITHCRC = CRC;
	I3Cx->MCONTROL.READTERMCNT   = 0;
	I3Cx->MTXB.RegValue = Command;		
	
	// write data to tx fifo to transmit.
	Nack = I3C_Write(I3Cx, I3C_COMTYPE_I3C_DDR, SlaveAddr, pTxData, TxByteNum);
	return Nack;

}









