#ifndef _GOWIN_I3C_H_
#define _GOWIN_I3C_H_

#include <stdint.h>


#ifdef __cplusplus
extern "C" {
#endif


#ifndef __I
  #ifdef __cplusplus
    #define   __I     volatile             /*!< Defines 'read only' permissions */
  #else
    #define   __I     volatile const       /*!< Defines 'read only' permissions */
  #endif
#endif

#ifndef __O
  #define     __O     volatile             /*!< Defines 'write only' permissions */
#endif

#ifndef __IO
  #define     __IO    volatile             /*!< Defines 'read / write' permissions */
#endif


#define MTXB_LAST (1 << 8)

// Master Interrupt Reg Bit Definition
#define MI_SSTART         (1 <<  8)
#define MI_MCONTROLFINISH (1 <<  9)
#define MI_COMCOMPLETE    (1 << 10)
#define MI_RFIFONOTEMPTY  (1 << 11)
#define MI_SFIFONOTFULL   (1 << 12)
#define MI_IBIRCV         (1 << 13)
#define MI_ERR            (1 << 15)

#define I3C_REQUEST_MASTER_STARTnADDR   (1)
#define I3C_REQUEST_MASTER_STOP         (2)
#define I3C_REQUEST_MASTER_IBI_RESP     (3)
#define I3C_REQUEST_MASTER_REQ_DAA      (4)
#define I3C_REQUEST_MASTER_STARTnSLVRST (5)
#define I3C_REQUEST_MASTER_HDR_EXIT     (6)

#define I3C_COMTYPE_I3C      (0)
#define I3C_COMTYPE_I2C      (1)
#define I3C_COMTYPE_I3C_DDR  (2)

#define I3C_IBISPTYPE_AUTO_ACK          (0)
#define I3C_IBISPTYPE_AUTO_NACK         (1)
#define I3C_IBISPTYPE_AUTO_ACK_SLV_RSP  (2)
#define I3C_IBISPTYPE_MANUAL_ACK        (3)

#define I3C_MANUAL_ACK_TYPE_ACK            (0)
#define I3C_MANUAL_ACK_TYPE_NACK           (1)
#define I3C_MANUAL_ACK_TYPE_ACK_WITH_MBYTE (2)


#define I3C_MSTE_IDLE              (0)
#define I3C_MSTE_SDA_SLV_LOW       (1)
#define I3C_MSTE_SDR_COM           (3)
#define I3C_MSTE_DDR_COM           (4)
#define I3C_MSTE_SETDA             (5)
#define I3C_MSTE_WAIT_MANUAL_RSP   (6)
#define I3C_MSTE_RCV_SLV_INT       (7)


#define I3C_MASTER_DIR_WRITE (0)
#define I3C_MASTER_DIR_READ  (1)

////////////////////////
//register definition

#pragma anon_unions 
typedef union _I3C_MCFG_RegDef_ {
	struct {
		uint32_t MENABLE         :  1;
		uint32_t RESERVED_2_1    :  2;
		uint32_t MDISTIMEOUT     :  1;
		uint32_t RESERVED_7_4    :  4;
		uint32_t PPHIGH          :  4;
		uint32_t PPLOWEXTRA      :  4;
		uint32_t ODSCL           :  8;
		uint32_t ODHIGHEQUALPP   :  1;
		uint32_t RESERVED_27_25  :  3;
		uint32_t I2CSCL          :  4;
	};
	uint32_t     RegValue;
} I3C_MCFG_RegDef;


typedef union _I3C_SCFG_RegDef_ {
	struct {
		uint32_t SENABLE         :  1;
		uint32_t SNACK           :  1;
		uint32_t MATCHSAORDASS   :  1;
		uint32_t ERRIGNORE       :  1;
		uint32_t RESERVED_7_4    :  4;
		uint32_t PIDTYPESELECT   :  1;
		uint32_t HJWAIT          :  1;
		uint32_t RESERVED_15_10  :  6;
		uint32_t PULLDOWNSDACNT  :  8;
		uint32_t RESERVED_24     :  1;
		uint32_t SA              :  7;
	};
	uint32_t     RegValue;
} I3C_SCFG_RegDef;


typedef union _I3C_SSTS_RegDef_ {
	struct {
		uint32_t STSBUSY         :  1;
		uint32_t STSMSG          :  1;
		uint32_t STSCCCAH        :  1;
		uint32_t STSREAD         :  1;
		uint32_t STSWRITE        :  1;
		uint32_t STSDAA          :  1;
		uint32_t RESERVED_6      :  1;
		uint32_t START           :  1;
		uint32_t MATCHEDBA       :  1;
		uint32_t MATCHEDSAORDA   :  1;
		uint32_t STOP            :  1;
		uint32_t RFIFONOTEMPTY   :  1;
		uint32_t SFIFONOTFULL    :  1;
		uint32_t DAVALID         :  1;
		uint32_t CCCRCV          :  1;
		uint32_t ERR             :  1;
		uint32_t RESERVED_16     :  1;
		uint32_t CCCAH           :  1;
		uint32_t DATANEED        :  1;
		uint32_t SLVRST          :  1;
		uint32_t REQUEST         :  1;
		uint32_t REQUESTACK      :  1;
		uint32_t RESERVED_31_22  : 10;
	};	
	uint32_t     RegValue;
} I3C_SSTS_RegDef;


typedef union _I3C_SCONTROL_RegDef_ {
	struct {
		uint32_t REQUEST         :  2;
		uint32_t RESERVED_7_2    :  6;
		uint32_t IBIMDATA        :  8;
		uint32_t RESERVED_31_16  : 16;
	};
	uint32_t     RegValue;
} I3C_SCONTROL_RegDef;


typedef union _I3C_SIS_RegDef_ {
	struct {
		uint32_t RESERVED_6_0    :  7;
		uint32_t START           :  1;
		uint32_t MATCHEDBA       :  1;
		uint32_t MATCHEDSAORDA   :  1;
		uint32_t STOP            :  1;
		uint32_t RFIFONOTEMPTY   :  1;
		uint32_t SFIFONOTFULL    :  1;
		uint32_t DAVALID         :  1;
		uint32_t CCCRCV          :  1;
		uint32_t ERR             :  1;
		uint32_t RESERVED_16     :  1;
		uint32_t CCCAH           :  1;
		uint32_t DATANEED        :  1;
		uint32_t SLVRST          :  1;
		uint32_t REQUEST         :  1;
		uint32_t RESERVED_31_21  : 11;
	};
	uint32_t     RegValue;
} I3C_SIS_RegDef;


typedef union _I3C_SIC_RegDef_ {
	struct {
		uint32_t RESERVED_6_0    :  7;
		uint32_t START           :  1;
		uint32_t MATCHEDBA       :  1;
		uint32_t MATCHEDSAORDA   :  1;
		uint32_t STOP            :  1;
		uint32_t RFIFONOTEMPTY   :  1;
		uint32_t SFIFONOTFULL    :  1;
		uint32_t DAVALID         :  1;
		uint32_t CCCRCV          :  1;
		uint32_t ERR             :  1;
		uint32_t RESERVED_16     :  1;
		uint32_t CCCAH           :  1;
		uint32_t DATANEED        :  1;
		uint32_t SLVRST          :  1;
		uint32_t REQUEST         :  1;
		uint32_t RESERVED_31_21  : 11;
	};
	uint32_t     RegValue;
} I3C_SIC_RegDef;


typedef union _I3C_SIM_RegDef_ {
	struct {
		uint32_t RESERVED_6_0    :  7;
		uint32_t START           :  1;
		uint32_t MATCHEDBA       :  1;
		uint32_t MATCHEDSAORDA   :  1;
		uint32_t STOP            :  1;
		uint32_t RFIFONOTEMPTY   :  1;
		uint32_t SFIFONOTFULL    :  1;
		uint32_t DAVALID         :  1;
		uint32_t CCCRCV          :  1;
		uint32_t ERR             :  1;
		uint32_t RESERVED_16     :  1;
		uint32_t CCCAH           :  1;
		uint32_t DATANEED        :  1;
		uint32_t SLVRST          :  1;
		uint32_t REQUEST         :  1;
		uint32_t RESERVED_31_21 : 11;
	};
	uint32_t     RegValue;
} I3C_SIM_RegDef;


typedef union _I3C_SERR_RegDef_ {
	struct {
		uint32_t OVERRCV         :  1;
		uint32_t RESERVED_1      :  1;
		uint32_t NACKWITHOUTDATA :  1;
		uint32_t RESERVED_7_3    :  5;
		uint32_t SDRPARERR       :  1;
		uint32_t RESERVED_10_9   :  2;
		uint32_t S0ORS1ERR       :  1;
		uint32_t RESERVED_15_12  :  4;
		uint32_t READEMPTY       :  1;
		uint32_t WRITEFULL       :  1;
		uint32_t RESERVED_31_18  : 14;
	};
	uint32_t     RegValue;
} I3C_SERR_RegDef;


typedef union _I3C_SDATACONTROL_RegDef_ {
	struct {
		uint32_t SFIFOCLR        :  1;
		uint32_t RFIFOCLR        :  1;
		uint32_t RESERVED_15_2   : 14;
		uint32_t SFIFOCNT        :  5;
		uint32_t RESERVED_23_21  :  3;
		uint32_t RFIFOCNT        :  5;
		uint32_t RESERVED_29     :  1;
		uint32_t SFIFOFULL       :  1;
		uint32_t RFIFOEMPTY      :  1;
	};
	uint32_t     RegValue;
} I3C_SDATACONTROL_RegDef;


typedef union _I3C_STXB_RegDef_ {
	struct {
		uint32_t WDATA           :  8;
		uint32_t LAST            :  1;
		uint32_t RESERVED_31_9   : 23;
	};
	uint32_t     RegValue;
} I3C_STXB_RegDef;


typedef union _I3C_SRXB_RegDef_ {
	struct {
		uint32_t RDATA           :  8;
		uint32_t RESERVED_31_9   : 24;
	};
	uint32_t     RegValue;
} I3C_SRXB_RegDef;


typedef union _I3C_SDA_RegDef_ {
	struct {
		uint32_t DAVALID         :  1;
		uint32_t DA              :  7;
		uint32_t RESERVED_31_8   : 24;
	};
	uint32_t     RegValue;
} I3C_SDA_RegDef;


typedef union _I3C_SVFVORRV_RegDef_ {
	struct {
		uint32_t VFVORRV;
	};
	uint32_t     RegValue;
}I3C_SVFVORRV_RegDef;


typedef union _I3C_SBCRANDDCR_RegDef_ {
	struct {
		uint32_t RESERVED_7_0    :  8;
		uint32_t DCR             :  8;
		uint32_t BCR             :  8;
		uint32_t RESERVED_31_24  :  8;
	};
	uint32_t     RegValue;
} I3C_SBCRANDDCR_RegDef;


typedef union _I3C_SMMID_RegDef_ {
	struct {
		uint32_t MMID            : 15;
		uint32_t RESERVED_31_15  : 17;
	};
	uint32_t         RegValue;
} I3C_SMMID_RegDef;


typedef union _I3C_MCONTROL_RegDef_ {
	struct {
		uint32_t REQUEST         :  3;
		uint32_t RESERVED_3      :  1;
		uint32_t COMTYPE         :  2;
		uint32_t IBIRSPTYPE      :  2;
		uint32_t DIRECTION       :  1;
		uint32_t COMADDR         :  7;
		uint32_t READTERMCNT     :  8;
		uint32_t DDRENDWITHCRC   :  1;
		uint32_t RESERVED_31_25  :  7;
	};
	uint32_t     RegValue;
} I3C_MCONTROL_RegDef; 


typedef union _I3C_MSTS_RegDef_ {
	struct {
		uint32_t MSTE            :  3;
		uint32_t RESERVED_3      :  1;
		uint32_t BWN             :  1;
		uint32_t NACK            :  1;
		uint32_t SRTYPE          :  2;
		uint32_t SSTART          :  1;
		uint32_t MCONTROLFINISH  :  1;
		uint32_t COMCOMPLETE     :  1;
		uint32_t RFIFONOTEMPTY   :  1;
		uint32_t SFIFONOTFULL    :  1;
		uint32_t IBIRCV          :  1;
		uint32_t RESERVED_14     :  1;
		uint32_t ERR             :  1;
		uint32_t RESERVED_23_16  :  8;
		uint32_t IBIADDRESS      :  7;
		uint32_t RESERVED_31     :  1;
	};
	uint32_t     RegValue;
} I3C_MSTS_RegDef;


typedef union _I3C_MIBIFORMCFG_RegDef_ {
	struct {
		uint32_t SADDRESS0       :  6;
		uint32_t SADDRESS1       :  6;
		uint32_t SADDRESS2       :  6;
		uint32_t SADDRESS3       :  6;
		uint32_t SADDRESS4       :  6;
		uint32_t DAMSB0          :  1;
		uint32_t NOIBIMBYTE      :  1;
	};
	uint32_t     RegValue;
} I3C_MIBIFORMCFG_RegDef;


typedef union _I3C_MIS_RegDef_ {
	struct {
		uint32_t RESERVED_7_0    :  8;
		uint32_t SSTART          :  1;
		uint32_t MCONTROLFINISH  :  1;
		uint32_t COMCOMPLETE     :  1;
		uint32_t RFIFONOTEMPTY   :  1;
		uint32_t SFIFONOTFULL    :  1;
		uint32_t IBIRCV          :  1;
		uint32_t RESERVED_14     :  1;
		uint32_t ERR             :  1;
		uint32_t RESERVED_31_16  : 16;
	};
	uint32_t     RegValue;
} I3C_MIS_RegDef;


typedef union _I3C_MIC_RegDef_ {
	struct {
		uint32_t RESERVED_7_0    :  8;
		uint32_t SSTART          :  1;
		uint32_t MCONTROLFINISH  :  1;
		uint32_t COMCOMPLETE     :  1;
		uint32_t RFIFONOTEMPTY   :  1;
		uint32_t SFIFONOTFULL    :  1;
		uint32_t IBIRCV          :  1;
		uint32_t RESERVED_14     :  1;
		uint32_t ERR             :  1;
		uint32_t RESERVED_31_16  : 16;
	};
	uint32_t     RegValue;
} I3C_MIC_RegDef;


typedef union _I3C_MIM_RegDef_ {
	struct {
		uint32_t RESERVED_7_0    :  8;
		uint32_t SSTART          :  1;
		uint32_t MCONTROLFINISH  :  1;
		uint32_t COMCOMPLETE     :  1;
		uint32_t RFIFONOTEMPTY   :  1;
		uint32_t SFIFONOTFULL    :  1;
		uint32_t IBIRCV          :  1;
		uint32_t RESERVED_14     :  1;
		uint32_t ERR             :  1;
		uint32_t RESERVED_31_16  : 16;
	};
	uint32_t     RegValue;
} I3C_MIM_RegDef;


typedef union _I3C_MERR_RegDef_ {
	struct {
		uint32_t RESERVED_1_0    :  2;
		uint32_t DAABANACK       :  1;
		uint32_t I2CWNACK        :  1;
		uint32_t RESERVED_8_4    :  5;
		uint32_t DDRFRAME        :  1;
		uint32_t DDRCRC5         :  1;
		uint32_t DDRCMDDATA      :  1;
		uint32_t RESERVED_15_12  :  4;
		uint32_t READEMPTY       :  1;
		uint32_t WRITEFULL       :  1;
		uint32_t RESERVED_18     :  1;
		uint32_t ERRREQUEST      :  1;
		uint32_t COMTIMEOUT      :  1;
		uint32_t RESERVED_31_21  : 11;
	};
	uint32_t     RegValue;
} I3C_MERR_RegDef;


typedef union _I3C_MDATACONTROL_RegDef_ {
	struct {
		uint32_t SFIFOCLR        :  1;
		uint32_t RFIFOCLR        :  1;
		uint32_t RESERVED_15_2   : 14;
		uint32_t SFIFOCNT        :  5;
		uint32_t RESERVED_23_21  :  3;
		uint32_t RFIFOCNT        :  5;
		uint32_t RESERVED_29     :  1;
		uint32_t SFIFOFULL       :  1;
		uint32_t RFIFOEMPTY      :  1;
	};
	uint32_t     RegValue;
} I3C_MDATACONTROL_RegDef;


typedef union _I3C_MTXB_RegDef_ {
	struct {
		uint32_t WDATA           :  8;
		uint32_t LAST            :  1;
		uint32_t RESERVED_31_9   : 23;
	};
	uint32_t     RegValue;
} I3C_MTXB_RegDef;


typedef union _I3C_MTXBE_RegDef_ {
	struct {
		uint32_t WDATA           :  8;
		uint32_t RESERVED_31_8   : 24;
	};
	uint32_t     RegValue;
} I3C_MTXBE_RegDef;


typedef union _I3C_MRXB_RegDef_ {
	struct {
		uint32_t RDATA           :  8;
		uint32_t RESERVED_31_8   : 24;
	};
	uint32_t     RegValue;
} I3C_MRXB_RegDef;


typedef union _I3C_DID_RegDef_ {
	struct {
		uint32_t CLOCKNUM        :  2;
		uint32_t ROLE            :  2;
		uint32_t FUNCTION        :  2;
		uint32_t VERSION         :  6;
		uint32_t RESERVED_31_12  : 20;
	};
	uint32_t     RegValue;
}I3C_DID_RegDef;




typedef struct _Gowin_I3C_TypeDef_ {
	__IO I3C_MCFG_RegDef          MCFG;           // 0x00, Master Configuration	
	__IO I3C_SCFG_RegDef          SCFG;           // 0x04, Slave Configuration
	__IO I3C_SSTS_RegDef          SSTS;           // 0x08, Slave Status
	__IO I3C_SCONTROL_RegDef      SCONTROL;       // 0x0c, Slave Control
	__IO I3C_SIS_RegDef           SIS;            // 0x10, Slave Interrupt Setting
	__O  I3C_SIC_RegDef           SIC;            // 0x14, Slave Interrupt Clear
	__I  I3C_SIM_RegDef           SIM;            // 0x18, Slave Interrupt Mask
	__IO I3C_SERR_RegDef          SERR;           // 0x1c, Slave Error
	__I  uint32_t                 RESERVED_20;    // 0x20, preserved
	__I  uint32_t                 RESERVED_24;    // 0x24, preserved
	__I  uint32_t                 RESERVED_28;    // 0x28, preserved
	__IO I3C_SDATACONTROL_RegDef  SDATACONTROL;   // 0x2c, Slave Data Control
	__O  I3C_STXB_RegDef          STXB;           // 0x30, Slave Tx Byte
	__I  uint32_t                 RESERVED_34;    // 0x34, preserved	
	__I  uint32_t                 RESERVED_38;    // 0x38, preserved	
	__I  uint32_t                 RESERVED_3c;    // 0x3c, preserved	
	__I  I3C_SRXB_RegDef          SRXB;           // 0x40, Slave Rx Byte
	__I  uint32_t                 RESERVED_44;    // 0x44, preserved	
	__I  uint32_t                 RESERVED_48;    // 0x48, preserved	
	__I  uint32_t                 RESERVED_4c;    // 0x4c, preserved	
	__I  uint32_t                 RESERVED_50;    // 0x50, preserved	
	__I  uint32_t                 RESERVED_54;    // 0x54, preserved	
	__I  uint32_t                 RESERVED_58;    // 0x58, preserved	
	__I  uint32_t                 RESERVED_5c;    // 0x5c, preserved	
	__I  uint32_t                 RESERVED_60;    // 0x60, preserved	
	__IO I3C_SDA_RegDef           SDA;            // 0x64, Slave Dynamic Address
	__I  uint32_t                 RESERVED_68;    // 0x68, preserved	
	__IO I3C_SVFVORRV_RegDef      SVFVORRV;       // 0x6c, Slave Vender Fixed Value or Random Value
	__IO I3C_SBCRANDDCR_RegDef    SBCRANDDCR;     // 0x70, Slave BCR and DCR
	__IO I3C_SMMID_RegDef         SMMID;          // 0x74, Slave MIMP Vender ID
	__I  uint32_t                 RESERVED_78;    // 0x78, preserved	
	__I  uint32_t                 RESERVED_7c;    // 0x7c, preserved	
	__I  uint32_t                 RESERVED_80;    // 0x80, preserved	
	__IO I3C_MCONTROL_RegDef      MCONTROL;       // 0x84, Master Control
	__IO I3C_MSTS_RegDef          MSTS;           // 0x88, Master Status
	__IO I3C_MIBIFORMCFG_RegDef   MIBIFORMCFG;    // 0x8c, Master IBI Format Configuration
	__IO I3C_MIS_RegDef           MIS;            // 0x90, Master Interrupt Setting
	__O  I3C_MIC_RegDef           MIC;            // 0x94, Master Interrupt Clear
	__I  I3C_MIM_RegDef           MIM;            // 0x98, Master Interrupt Mask
	__IO I3C_MERR_RegDef          MERR;           // 0x9c, Master Error
	__I  uint32_t                 RESERVED_a0;    // 0xa0, preserved	
	__I  uint32_t                 RESERVED_a4;    // 0xa4, preserved	
	__I  uint32_t                 RESERVED_a8;    // 0xa8, preserved		
	__IO I3C_MDATACONTROL_RegDef  MDATACONTROL;   // 0xac, Master Data Control
	__O  I3C_MTXB_RegDef          MTXB;           // 0xb0, Master Tx Byte
	__O  I3C_MTXBE_RegDef         MTXBE;          // 0xb4, Master Tx Byte Eventual
	__I  uint32_t                 RESERVED_b8;    // 0xb8, preserved		
	__I  uint32_t                 RESERVED_bc;    // 0xbc, preserved		
	__I  I3C_MRXB_RegDef          MRXB;           // 0xc0, Master Rx Byte
	__I  I3C_DID_RegDef           DID;            // 0xc4, Device ID
	
} Gowin_I3C_TypeDef;



//#define I3C0              ((Gowin_I3C_TypeDef *)AHB2PERIPH_BASE)
#define I3C0              ((Gowin_I3C_TypeDef *)APB2MASTER1_BASE)




void gowin_i3c_init(Gowin_I3C_TypeDef * I3Cx);
void gowin_i3c_clear_sfifo(Gowin_I3C_TypeDef * I3Cx);
void gowin_i3c_clear_rfifo(Gowin_I3C_TypeDef * I3Cx);
uint8_t gowin_i3c_get_sfifo_cnt(Gowin_I3C_TypeDef * I3Cx);
uint8_t gowin_i3c_get_rfifo_cnt(Gowin_I3C_TypeDef * I3Cx);

void gowin_i3c_set_IBIRspType(Gowin_I3C_TypeDef * I3Cx, uint8_t IBIRspType);


void I3C_Wait_MControl_Finish(Gowin_I3C_TypeDef *I3Cx);
void I3C_Wait_Com_Complete(Gowin_I3C_TypeDef *I3Cx);
void I3C_IBI_Manual_Ack(Gowin_I3C_TypeDef *I3Cx, uint8_t ManualAckType);


uint8_t I3C_Write(Gowin_I3C_TypeDef *I3Cx, uint8_t ComType, uint8_t SlaveAddr, uint8_t *pTxData, uint32_t DataByteLen);
uint8_t I3C_Read(Gowin_I3C_TypeDef *I3Cx, uint8_t ComType, uint8_t SlaveAddr, uint8_t *pRxData, uint32_t DataByteLen);
uint8_t I3C_ReadRxData(Gowin_I3C_TypeDef *I3Cx, uint8_t *pRxData, uint32_t RxDataNum);

void I3C_SendStop(Gowin_I3C_TypeDef *I3Cx);

uint8_t I3C_Write_Slave_Reg(Gowin_I3C_TypeDef *I3Cx, uint8_t ComType, uint8_t SlaveAddr, uint8_t RegAddr, uint8_t TxData);
uint8_t I3C_Read_Slave_Reg(Gowin_I3C_TypeDef *I3Cx, uint8_t ComType, uint8_t SlaveAddr, uint8_t RegAddr, uint8_t *pRxData);


void I3C_ExitDDR(Gowin_I3C_TypeDef *I3Cx, uint8_t SlaveAddr);
uint8_t I3C_DDR_Read(Gowin_I3C_TypeDef *I3Cx, uint8_t SlaveAddr, uint8_t Command, uint8_t *pRxData, uint8_t RxByteNum, uint8_t CRC);
uint8_t I3C_DDR_Write(Gowin_I3C_TypeDef *I3Cx, uint8_t SlaveAddr, uint8_t Command, uint8_t *pTxData, uint8_t TxByteNum, uint8_t CRC);





#ifdef __cplusplus
}
#endif

#endif




