
#include "gowin_i3c_ccc.h"
#include "gw1ns4c.h"
#include <stdio.h>




uint8_t I3C_CCC_Broadcast(Gowin_I3C_TypeDef *I3Cx, uint8_t CCC_Code, uint8_t *pDefByte, uint8_t *pData, uint32_t DataLen)
{
	uint32_t i;
	uint8_t Nack = 0;
	volatile I3C_MCONTROL_RegDef MControlBuf;
	
	MControlBuf.RegValue    = I3Cx->MCONTROL.RegValue;
	MControlBuf.REQUEST     = I3C_REQUEST_MASTER_STARTnADDR;
	MControlBuf.COMTYPE     = I3C_COMTYPE_I3C;
	MControlBuf.DIRECTION   = I3C_MASTER_DIR_WRITE;
	MControlBuf.COMADDR     = 0x7e;
	MControlBuf.READTERMCNT = 0;	
	I3Cx->MCONTROL.RegValue = MControlBuf.RegValue;			
	I3C_Wait_MControl_Finish(I3Cx);
	if(I3Cx->MSTS.NACK) {
		return 1;
	}	
	
	I3Cx->MTXB.RegValue = (pDefByte == NULL && pData == NULL) ? (CCC_Code | MTXB_LAST) : CCC_Code;
	if(pDefByte != NULL) {
		I3Cx->MTXB.RegValue = (pData == NULL) ? ((*pDefByte) | MTXB_LAST) : (*pDefByte);
	}
	
	if(DataLen > 0) {
		for(i = 0; i < DataLen - 1; i++) {
			I3Cx->MTXB.RegValue = *pData;
			pData++;
		}
		I3Cx->MTXBE.RegValue = (*pData);
	}
	I3C_Wait_Com_Complete(I3Cx);
	Nack = I3Cx->MSTS.NACK;
		
	// stop
	I3C_SendStop(I3Cx);	
	
	return Nack;
}


uint8_t I3C_CCC_Direct_Header(Gowin_I3C_TypeDef *I3Cx, uint8_t CCC_Code, uint8_t *pDefByte)
{
	uint8_t Nack;
	uint8_t Buf[2];
	uint32_t DataCnt = 1;
	Buf[0] = CCC_Code;
	if(pDefByte != NULL) {
		Buf[1] = *pDefByte;
		DataCnt++;
	}

	Nack = I3C_Write(I3Cx, I3C_COMTYPE_I3C, 0x7e, Buf, DataCnt);
	return Nack;
}


uint8_t I3C_CCC_Direct_WriteData(Gowin_I3C_TypeDef *I3Cx, uint8_t SlaveAddr, uint8_t *pData, uint32_t DataLen)
{
	uint32_t i = 0;
	
	I3Cx->MCONTROL.COMTYPE     = I3C_COMTYPE_I3C;
	I3Cx->MCONTROL.DIRECTION   = I3C_MASTER_DIR_WRITE;
	I3Cx->MCONTROL.COMADDR     = SlaveAddr;
	I3Cx->MCONTROL.READTERMCNT = 0;
	I3Cx->MCONTROL.REQUEST     = I3C_REQUEST_MASTER_STARTnADDR;
	
	I3C_Wait_MControl_Finish(I3Cx);
	if(I3Cx->MSTS.NACK) {
		return 1;
	}

	if(DataLen > 0) {
		for(i = 0; i < DataLen - 1; i++) {
			I3Cx->MTXB.RegValue = *pData;
			pData++;
		}
		I3Cx->MTXBE.RegValue = (*pData);
		I3C_Wait_Com_Complete(I3Cx);
	}

	return 0;
}


uint8_t I3C_CCC_Direct_ReadData(Gowin_I3C_TypeDef *I3Cx, uint8_t SlaveAddr, uint8_t *pData, uint32_t DataLen)
{
	uint8_t Nack;		
	Nack = I3C_Read(I3Cx, I3C_COMTYPE_I3C, SlaveAddr, pData, DataLen);
		
	return Nack;
}


uint8_t I3C_CCC_ENTDAA(Gowin_I3C_TypeDef *I3Cx, uint8_t *pDynamicAddr, uint32_t SlvNum, I3C_ENTDAA_RdBk *pEntDaaRdBkBuf)
{
	int SlvCnt = 0;
	uint8_t *pBuf = NULL;

	I3Cx->MCONTROL.RegValue = 0x0000fc04;
	I3C_Wait_MControl_Finish(I3Cx);
	if(I3Cx->MSTS.NACK) {
			return 1;
	}	
	while(1) {
		pBuf = (uint8_t *)(pEntDaaRdBkBuf);
		I3C_ReadRxData(I3Cx, pBuf, 8);
		I3Cx->MTXB.WDATA = (*pDynamicAddr) << 1;
		I3Cx->MCONTROL.RegValue = 0x0000fc04;
		I3C_Wait_MControl_Finish(I3Cx);
		if(I3Cx->MSTS.NACK) {
			return (SlvCnt != SlvNum);
		}	
		pDynamicAddr++;
		SlvCnt++;
		pEntDaaRdBkBuf++;
		if(I3Cx->MSTS.MSTE != 5) {
			break;
		}
		else if(SlvCnt == SlvNum) {
			return 0xff;
		}
		
	}
	return 0;
}


uint8_t I3C_CCC_SETDASA(Gowin_I3C_TypeDef *I3Cx, uint8_t *pSlvStaticAddr, uint8_t *pSlvDynamicAddr, uint32_t SlvNum)
{
	int i;
	uint8_t Nack = 0;
	
	volatile I3C_MCONTROL_RegDef MControlBuf;
	MControlBuf.RegValue    = I3Cx->MCONTROL.RegValue;
	MControlBuf.REQUEST     = I3C_REQUEST_MASTER_STARTnADDR;
	MControlBuf.COMTYPE     = I3C_COMTYPE_I3C;
	MControlBuf.DIRECTION   = I3C_MASTER_DIR_WRITE;
	MControlBuf.READTERMCNT = 0;	
	
	I3C_CCC_Direct_Header(I3Cx, I3C_CCC_CODE_SETDASA, NULL);
	
	for(i = 0; i < SlvNum; i++) {	
		MControlBuf.COMADDR     = *pSlvStaticAddr;
		I3Cx->MCONTROL.RegValue = MControlBuf.RegValue;
		I3C_Wait_MControl_Finish(I3Cx);
		if(I3Cx->MSTS.NACK) {
			return 1;
		}
		I3Cx->MTXBE.RegValue = (*pSlvDynamicAddr) << 1;
		I3C_Wait_Com_Complete(I3Cx);
		if(I3Cx->MSTS.NACK) {
			return 1;
		}
		pSlvStaticAddr++;
		pSlvDynamicAddr++;
	}
		
	I3C_SendStop(I3Cx);
	
	return Nack;
}


uint8_t I3C_CCC_SETNEWDA(Gowin_I3C_TypeDef *I3Cx, uint8_t *pSlvOldDA, uint8_t *pSlvNewDA, uint32_t SlvNum)
{
	int i;
	uint8_t Nack = 0;
	
	volatile I3C_MCONTROL_RegDef MControlBuf;
	MControlBuf.RegValue    = I3Cx->MCONTROL.RegValue;
	MControlBuf.REQUEST     = I3C_REQUEST_MASTER_STARTnADDR;
	MControlBuf.COMTYPE     = I3C_COMTYPE_I3C;
	MControlBuf.DIRECTION   = I3C_MASTER_DIR_WRITE;
	MControlBuf.READTERMCNT = 0;
	
	I3C_CCC_Direct_Header(I3Cx, I3C_CCC_CODE_SETNEWDA, NULL);
	
	for(i = 0; i < SlvNum; i++) {
		MControlBuf.COMADDR     = *pSlvOldDA;
		I3Cx->MCONTROL.RegValue = MControlBuf.RegValue;
		I3C_Wait_MControl_Finish(I3Cx);
		if(I3Cx->MSTS.NACK) {
			return 1;
		}
		I3Cx->MTXBE.RegValue = (*pSlvNewDA) << 1;
		I3C_Wait_Com_Complete(I3Cx);
		if(I3Cx->MSTS.NACK) {
			return 1;
		}
		pSlvOldDA++;
		pSlvNewDA++;
	}
	I3C_SendStop(I3Cx);
	return Nack;
}


uint8_t I3C_CCC_SETGRPA(Gowin_I3C_TypeDef *I3Cx, uint8_t *pSlvAddr, uint8_t GrpAddr, uint8_t SlvNum)
{
	int i;
	uint8_t Nack;
	uint8_t GrpAddrData;
	
	GrpAddrData = GrpAddr << 1;
	I3C_CCC_Direct_Header(I3Cx, I3C_CCC_CODE_DIRECT_SETGRPA, NULL);
	
	for(i = 0; i < SlvNum; i++) {
		Nack = I3C_CCC_Direct_WriteData(I3Cx, *pSlvAddr, &GrpAddrData, 1);
		pSlvAddr++;
	}
	I3C_SendStop(I3Cx);	
	return Nack;	
}


uint8_t I3C_CCC_RSTGRPA(Gowin_I3C_TypeDef *I3Cx, uint8_t *pSlvAddr, uint8_t SlvNum)
{
	int i;
	uint8_t Nack;
	I3C_CCC_Direct_Header(I3Cx, I3C_CCC_CODE_DIRECT_RSTGRPA, NULL);
	for(i = 0; i < SlvNum; i++) {
		Nack = I3C_CCC_Direct_WriteData(I3Cx, *pSlvAddr, NULL, 0);
		pSlvAddr++;
	}
	I3C_SendStop(I3Cx);
	return Nack;
}


uint8_t I3C_CCC_RSTACT_Write(Gowin_I3C_TypeDef *I3Cx, uint8_t *pSlvAddr, uint8_t DefByte, uint8_t SlvNum)
{
	int i;
	uint8_t Nack = 0;
	I3C_CCC_Direct_Header(I3Cx, I3C_CCC_CODE_DIRECT_RSTACT, &DefByte);
	for(i = 0; i < SlvNum; i++) {
		Nack = I3C_CCC_Direct_WriteData(I3Cx, *pSlvAddr, NULL, 0);
		pSlvAddr++;
	}
	I3C_SendStop(I3Cx);	
	return Nack;		
}


uint8_t I3C_CCC_RSTACT_Read(Gowin_I3C_TypeDef *I3Cx, uint8_t *pSlvAddr, uint8_t DefByte, uint8_t *pRetData, uint32_t SlvNum)
{
	int i;
	uint8_t Nack = 0;
	I3C_CCC_Direct_Header(I3Cx, I3C_CCC_CODE_DIRECT_RSTACT, &DefByte);
	for(i = 0; i < SlvNum; i++) {
		Nack = I3C_CCC_Direct_ReadData(I3Cx, *pSlvAddr, pRetData, 1);		
		pSlvAddr++;
		pRetData++;
	}
	
	I3C_SendStop(I3Cx);	
	return Nack;	
}


uint8_t I3C_CCC_SETXTIME(Gowin_I3C_TypeDef *I3Cx, uint8_t *pSlvAddr, uint8_t *pSubCmdByte, uint8_t **ppData, uint32_t *pDataLen, uint8_t SlvNum)
{
	int i;
	uint8_t Nack = 0;
	I3C_CCC_Direct_Header(I3Cx, I3C_CCC_CODE_DIRECT_SETXTIME, NULL);
	for(i = 0; i < SlvNum; i++) {
		I3Cx->MTXB.RegValue = *pSubCmdByte;
		Nack = I3C_CCC_Direct_WriteData(I3Cx, *pSlvAddr, *ppData, *pDataLen);
		pSlvAddr++;
		pSubCmdByte++;
		ppData++;
		pDataLen++;
	}
	I3C_SendStop(I3Cx);	
	return Nack;	
}


uint8_t I3C_CCC_SETXTIME_SingleSlave(Gowin_I3C_TypeDef *I3Cx, uint8_t SlvAddr, uint8_t SubCmdByte, uint8_t *pData, uint32_t DataLen)
{
	uint8_t Nack = 0;
	I3C_CCC_Direct_Header(I3Cx, I3C_CCC_CODE_DIRECT_SETXTIME, NULL);
	I3Cx->MTXB.RegValue = SubCmdByte;
	Nack = I3C_CCC_Direct_WriteData(I3Cx, SlvAddr, pData, DataLen);
	I3C_SendStop(I3Cx);	
	return Nack;	
}


uint8_t I3C_CCC_GETXTIME(Gowin_I3C_TypeDef *I3Cx, uint8_t *pSlvAddr, TimingSupportInfo *pTimingSupportInfo, uint32_t SlvNum)
{
	uint32_t i;
	uint8_t Nack = 0;
	I3C_CCC_Direct_Header(I3Cx, I3C_CCC_CODE_DIRECT_GETXTIME, NULL);
	
	for(i = 0; i < SlvNum; i++) {
		Nack = I3C_CCC_Direct_ReadData(I3Cx, *pSlvAddr, (uint8_t *)pTimingSupportInfo, sizeof(TimingSupportInfo));
		pSlvAddr++;
		pTimingSupportInfo++;
		
	}	
	I3C_SendStop(I3Cx);	
	return Nack;	
}


uint8_t I3C_CCC_GETXTIME_SingleSlave(Gowin_I3C_TypeDef *I3Cx, uint8_t SlvAddr, TimingSupportInfo *pTimingSupportInfo)
{
	uint8_t Nack = 0;
	I3C_CCC_Direct_Header(I3Cx, I3C_CCC_CODE_DIRECT_GETXTIME, NULL);
	Nack = I3C_CCC_Direct_ReadData(I3Cx, SlvAddr, (uint8_t *)pTimingSupportInfo, sizeof(TimingSupportInfo));
	I3C_SendStop(I3Cx);	
	return Nack;	
}


uint8_t I3C_CCC_SETMWL(Gowin_I3C_TypeDef *I3Cx, uint8_t *pSlvAddr, uint16_t *pLength, uint8_t SlvNum)
{
	int i;
	uint8_t Nack = 0;
	uint8_t Buf[2];
	I3C_CCC_Direct_Header(I3Cx, I3C_CCC_CODE_DIRECT_SETMWL, NULL);
	
	for(i = 0; i < SlvNum; i++) {
		Buf[0] = *pLength >> 8;
		Buf[1] = *pLength & 0xff;
		Nack = I3C_CCC_Direct_WriteData(I3Cx, *pSlvAddr, Buf, 2);
		if(Nack) 
			break;
		pSlvAddr++;
		pLength++;
	}
		
	I3C_SendStop(I3Cx);	
	return Nack;	
}


uint8_t I3C_CCC_GETMWL(Gowin_I3C_TypeDef *I3Cx, uint8_t *pSlvAddr, uint16_t *pLength, uint8_t SlvNum)
{
	int i;
	uint8_t Nack = 0;
	uint8_t Buf[2];
	I3C_CCC_Direct_Header(I3Cx, I3C_CCC_CODE_DIRECT_GETMWL, NULL);
	
	for(i = 0; i < SlvNum; i++) {
		Nack = I3C_CCC_Direct_ReadData(I3Cx, *pSlvAddr, Buf, 2);
		if(Nack) 
			break;
		*pLength = (Buf[0] << 8) + Buf[1];
		pSlvAddr++;
		pLength++;
	}
		
	I3C_SendStop(I3Cx);	
	return Nack;	
}


uint8_t I3C_CCC_SETMRL(Gowin_I3C_TypeDef *I3Cx, uint8_t *pSlvAddr, uint16_t *pLength, uint8_t *pIBIpayloadsize, uint8_t SlvNum)
{
	int i;
	uint8_t Nack = 0;
	uint8_t Buf[3];
	I3C_CCC_Direct_Header(I3Cx, I3C_CCC_CODE_DIRECT_SETMRL, NULL);
	
	for(i = 0; i < SlvNum; i++) {
		Buf[0] = *pLength >> 8;
		Buf[1] = *pLength & 0xff;
		Buf[2] = *pIBIpayloadsize;
		Nack = I3C_CCC_Direct_WriteData(I3Cx, *pSlvAddr, Buf, 3);
		pSlvAddr++;
		pLength++;
		pIBIpayloadsize++;
	}
		
	I3C_SendStop(I3Cx);	
	return Nack;	
}


uint8_t I3C_CCC_GETMRL(Gowin_I3C_TypeDef *I3Cx, uint8_t *pSlvAddr, uint16_t *pLength, uint8_t *pIBIpayloadsize, uint8_t SlvNum)
{
	int i;
	uint8_t Nack = 0;
	uint8_t Buf[3];
	I3C_CCC_Direct_Header(I3Cx, I3C_CCC_CODE_DIRECT_GETMRL, NULL);
	
	for(i = 0; i < SlvNum; i++) {
		Nack = I3C_CCC_Direct_ReadData(I3Cx, *pSlvAddr, Buf, 3);
//		if(Nack) 
//			break;
		*pLength = (Buf[0] << 8) + Buf[1];
		*pIBIpayloadsize = Buf[2];
		pSlvAddr++;
		pLength++;
		pIBIpayloadsize++;
	}
		
	I3C_SendStop(I3Cx);	
	return Nack;	
}


void I3C_CCC_ENEC_DISEC_Direct(Gowin_I3C_TypeDef *I3Cx, uint8_t CCC_Code, uint8_t *pSlvAddr, uint8_t *pSlvEvtByte, uint32_t SlvNum)
{
	int i;
	I3C_CCC_Direct_Header(I3Cx, CCC_Code, NULL);
	
	for(i = 0; i < SlvNum; i++) {
		I3C_CCC_Direct_WriteData(I3Cx, *pSlvAddr, pSlvEvtByte, 1);
		pSlvAddr++;
		pSlvEvtByte++;
	}
	
	I3C_SendStop(I3Cx);
	return;
}


void I3C_CCC_ENTASx_Direct(Gowin_I3C_TypeDef *I3Cx, uint8_t CCC_Code, uint8_t *pSlvAddr, uint32_t SlvNum)
{
	int i;
	uint8_t tmp[2] = {0, 0};
	uint8_t *pTmp = tmp;
	I3C_CCC_Direct_Header(I3Cx, CCC_Code, NULL);
	for(i = 0; i < SlvNum; i++) {
		I3C_CCC_Direct_WriteData(I3Cx, *pSlvAddr, NULL, 0);
		pSlvAddr++;
		pTmp++;
	}
	
	I3C_SendStop(I3Cx);
	return;	
}


void I3C_CCC_GETBCR(Gowin_I3C_TypeDef *I3Cx, uint8_t *pSlvAddr, uint8_t *pBCRByte, uint32_t SlvNum)
{
	int i;
	I3C_CCC_Direct_Header(I3Cx, I3C_CCC_CODE_GETBCR, NULL);
	
	for(i = 0; i < SlvNum; i++) {
		I3C_CCC_Direct_ReadData(I3Cx, *pSlvAddr, pBCRByte, 1);
		pSlvAddr++;
		pBCRByte++;
	}
	
	I3C_SendStop(I3Cx);
	
	return;
}


void I3C_CCC_GETDCR(Gowin_I3C_TypeDef *I3Cx, uint8_t *pSlvAddr, uint8_t *pDCRByte, uint32_t SlvNum)
{
	int i;
	I3C_CCC_Direct_Header(I3Cx, I3C_CCC_CODE_GETDCR, NULL);
	
	for(i = 0; i < SlvNum; i++) {
		I3C_CCC_Direct_ReadData(I3Cx, *pSlvAddr, pDCRByte, 1);
		pSlvAddr++;
		pDCRByte++;
	}
	
	I3C_SendStop(I3Cx);
	
	return;
}


void I3C_CCC_GETPID(Gowin_I3C_TypeDef *I3Cx, uint8_t *pSlvAddr, uint64_t *pPIDByte, uint32_t SlvNum)
{
	int i;
	uint8_t PIDBuf[6];
	I3C_CCC_Direct_Header(I3Cx, I3C_CCC_CODE_GETPID, NULL);
	
	for(i = 0; i < SlvNum; i++) {
		I3C_CCC_Direct_ReadData(I3Cx, *pSlvAddr, PIDBuf, 6);
		*pPIDByte = get6ByteID(PIDBuf);
		pSlvAddr++;
		pPIDByte++;
	}
		
	I3C_SendStop(I3Cx);

	return;
}


void I3C_CCC_GETSTATUS(Gowin_I3C_TypeDef *I3Cx, uint8_t *pSlvAddr, uint16_t *pStatus, uint32_t SlvNum)
{
	int i;
	uint8_t StatusBuf[2];
	I3C_CCC_Direct_Header(I3Cx, I3C_CCC_CODE_GETSTATUS, NULL);

	for(i = 0; i < SlvNum; i++) {
		I3C_CCC_Direct_ReadData(I3Cx, *pSlvAddr, StatusBuf, 2);
		*pStatus = (StatusBuf[0] << 8) + StatusBuf[1];
		pSlvAddr++;
		pStatus++;
	}
	
	I3C_SendStop(I3Cx);
	
	return;
}






uint64_t get6ByteID(uint8_t *pUniqueIDBuf)
{
	int i;
	uint64_t tmp = 0;
	uint8_t *ptmp = (uint8_t *)(&tmp);
	for(i = 0; i < 6; i++) {
		*ptmp = pUniqueIDBuf[5 - i];
		ptmp++;
	}
	return tmp;
}


void PrintENTDAA_RdBk(I3C_ENTDAA_RdBk *pEntDaaRdBkBuf)
{
	int i;
	uint64_t UniqueID;
	UniqueID = get6ByteID(pEntDaaRdBkBuf->UniqueID);
	printf("Unique ID: %012llx\r\n", UniqueID);
	for(i = 0; i < 6; i++) {
		printf("%02x ", pEntDaaRdBkBuf->UniqueID[i]);
	}
	printf("\r\n");
	
	printf("BCR: %02x\r\n", pEntDaaRdBkBuf->BCR_Value);
	printf("  MaxDataSpeedLimitation  : %x\r\n", pEntDaaRdBkBuf->MaxDataSpeedLimitation);
	printf("  IBI_RequestCapable      : %x\r\n", pEntDaaRdBkBuf->IBI_RequestCapable);
	printf("  IBI_Payload             : %x\r\n", pEntDaaRdBkBuf->IBI_Payload);
	printf("  OfflineCapable          : %x\r\n", pEntDaaRdBkBuf->OfflineCapable);
	printf("  VirtualSlaveSupport     : %x\r\n", pEntDaaRdBkBuf->VirtualSlaveSupport);
	printf("  AdvancedCapabilities    : %x\r\n", pEntDaaRdBkBuf->AdvancedCapabilities);
	printf("  DeviceRole              : %x\r\n", pEntDaaRdBkBuf->DeviceRole);
	
	printf("DCR: %02x\r\n", pEntDaaRdBkBuf->DCR_Value);
	
	return;
}




