#ifndef _GOWIN_I3C_CCC_DEFINES_H_
#define _GOWIN_I3C_CCC_DEFINES_H_

#include <stdint.h>
#include "gowin_i3c.h"

// macro defines of Common Command Codes (CCC)

#define I3C_CCC_CODE_BROADCAST_ENEC      (0x00)
#define I3C_CCC_CODE_BROADCAST_DISEC     (0x01)
#define I3C_CCC_CODE_BROADCAST_ENTAS0    (0x02)
#define I3C_CCC_CODE_BROADCAST_ENTAS1    (0x03)
#define I3C_CCC_CODE_BROADCAST_ENTAS2    (0x04)
#define I3C_CCC_CODE_BROADCAST_ENTAS3    (0x05)
#define I3C_CCC_CODE_BROADCAST_SETMWL    (0x09)
#define I3C_CCC_CODE_BROADCAST_SETMRL    (0x0a)
#define I3C_CCC_CODE_BROADCAST_ENTTM     (0x0b)


#define I3C_CCC_CODE_BROADCAST_SETXTIME  (0x28)
#define I3C_CCC_CODE_BROADCAST_RSTACT    (0x2a)

#define I3C_CCC_CODE_BROADCAST_RSTDAA    (0x86)
#define I3C_CCC_CODE_BROADCAST_SETAASA   (0x29)

#define I3C_CCC_CODE_DIRECT_ENEC         (0x80)
#define I3C_CCC_CODE_DIRECT_DISEC        (0x81)
#define I3C_CCC_CODE_DIRECT_ENTAS1       (0x83)
#define I3C_CCC_CODE_DIRECT_ENTAS2       (0x84)
#define I3C_CCC_CODE_DIRECT_ENTAS3       (0x85)

#define I3C_CCC_CODE_DIRECT_SETMWL       (0x89)
#define I3C_CCC_CODE_DIRECT_SETMRL       (0x8a)
#define I3C_CCC_CODE_DIRECT_GETMWL       (0x8b)
#define I3C_CCC_CODE_DIRECT_GETMRL       (0x8c)

#define I3C_CCC_CODE_DIRECT_SETXTIME     (0x98)
#define I3C_CCC_CODE_DIRECT_GETXTIME     (0x99)
#define I3C_CCC_CODE_DIRECT_RSTACT       (0x9a)

#define I3C_CCC_CODE_DIRECT_SETGRPA      (0x9b)
#define I3C_CCC_CODE_DIRECT_RSTGRPA      (0x9c)
#define I3C_CCC_CODE_DIRECT_MLANE        (0x9d)

#define I3C_CCC_CODE_RSTDAA    (0x86)
#define I3C_CCC_CODE_SETDASA   (0x87)
#define I3C_CCC_CODE_SETNEWDA  (0x88)


#define I3C_CCC_CODE_GETPID    (0x8d)
#define I3C_CCC_CODE_GETBCR    (0x8e)
#define I3C_CCC_CODE_GETDCR    (0x8f)
#define I3C_CCC_CODE_GETSTATUS (0x90)
#define I3C_CCC_CODE_GETACCMST (0x91)

#ifdef __cplusplus
extern "C" {
#endif

#pragma anon_unions 
typedef struct _I3C_ENTDAA_RdBk_ {
	uint8_t UniqueID[6];
	union {
		struct {
			uint8_t MaxDataSpeedLimitation : 1;
			uint8_t IBI_RequestCapable : 1;
			uint8_t IBI_Payload : 1;
			uint8_t OfflineCapable: 1;
			uint8_t VirtualSlaveSupport : 1;
			uint8_t AdvancedCapabilities : 1;
			uint8_t DeviceRole : 2;
		};
		uint8_t BCR_Value;
	};	
	uint8_t DCR_Value;
} I3C_ENTDAA_RdBk;


typedef union _SlaveEventsCmdByteDef_ {
	struct {
		uint8_t ENINT        : 1;
		uint8_t ENMR         : 1;
		uint8_t RESERVED_2   : 1;
		uint8_t ENHJ         : 1;
		uint8_t RESERVED_7_4 : 4;
	};
	uint8_t ByteValue;
} SlaveEventsCmdByteDef;

typedef struct _TimingSupportInfo_ {
	uint8_t SupportedModes;
	uint8_t State;
	uint8_t Frequency;
	uint8_t Inaccuracy;
} TimingSupportInfo;


uint8_t I3C_CCC_Broadcast(Gowin_I3C_TypeDef *I3Cx, uint8_t CCC_Code, uint8_t *pDefByte, uint8_t *pData, uint32_t DataLen);


uint8_t I3C_CCC_Direct_Header(Gowin_I3C_TypeDef *I3Cx, uint8_t CCC_Code, uint8_t *pDefByte);
uint8_t I3C_CCC_Direct_WriteData(Gowin_I3C_TypeDef *I3Cx, uint8_t SlaveAddr, uint8_t *pData, uint32_t DataLen);
uint8_t I3C_CCC_Direct_ReadData(Gowin_I3C_TypeDef *I3Cx, uint8_t SlaveAddr, uint8_t *pData, uint32_t DataLen);

uint8_t I3C_CCC_ENTDAA(Gowin_I3C_TypeDef *I3Cx, uint8_t *pDynamicAddr, uint32_t SlvNum, I3C_ENTDAA_RdBk *pEntDaaRdBkBuf);


uint8_t I3C_CCC_SETDASA(Gowin_I3C_TypeDef *I3Cx, uint8_t *pSlvStaticAddr, uint8_t *pSlvDynamicAddr, uint32_t SlvNum);
uint8_t I3C_CCC_SETNEWDA(Gowin_I3C_TypeDef *I3Cx, uint8_t *pSlvOldDA, uint8_t *pSlvNewDA, uint32_t SlvNum);

void I3C_CCC_ENEC_DISEC_Direct(Gowin_I3C_TypeDef *I3Cx, uint8_t CCC_Code, uint8_t *pSlvAddr, uint8_t *pSlvEvtByte, uint32_t SlvNum);
void I3C_CCC_ENTASx_Direct(Gowin_I3C_TypeDef *I3Cx, uint8_t CCC_Code, uint8_t *pSlvAddr, uint32_t SlvNum);

uint8_t I3C_CCC_SETMWL(Gowin_I3C_TypeDef *I3Cx, uint8_t *pSlvAddr, uint16_t *pLength, uint8_t SlvNum);
uint8_t I3C_CCC_GETMWL(Gowin_I3C_TypeDef *I3Cx, uint8_t *pSlvAddr, uint16_t *pLength, uint8_t SlvNum);


uint8_t I3C_CCC_SETMRL(Gowin_I3C_TypeDef *I3Cx, uint8_t *pSlvAddr, uint16_t *pLength, uint8_t *pIBIpayloadsize, uint8_t SlvNum);
uint8_t I3C_CCC_GETMRL(Gowin_I3C_TypeDef *I3Cx, uint8_t *pSlvAddr, uint16_t *pLength, uint8_t *pIBIpayloadsize, uint8_t SlvNum);

void I3C_CCC_GETBCR(Gowin_I3C_TypeDef *I3Cx, uint8_t *pSlvAddr, uint8_t *pBCRByte, uint32_t SlvNum);
void I3C_CCC_GETDCR(Gowin_I3C_TypeDef *I3Cx, uint8_t *pSlvAddr, uint8_t *pDCRByte, uint32_t SlvNum);
void I3C_CCC_GETPID(Gowin_I3C_TypeDef *I3Cx, uint8_t *pSlvAddr, uint64_t *pPIDByte, uint32_t SlvNum);

void I3C_CCC_GETSTATUS(Gowin_I3C_TypeDef *I3Cx, uint8_t *pSlvAddr, uint16_t *pStatus, uint32_t SlvNum);

uint8_t I3C_CCC_SETXTIME(Gowin_I3C_TypeDef *I3Cx, uint8_t *pSlvAddr, uint8_t *pSubCmdByte, uint8_t **ppData, uint32_t *pDataLen, uint8_t SlvNum);
uint8_t I3C_CCC_SETXTIME_SingleSlave(Gowin_I3C_TypeDef *I3Cx, uint8_t SlvAddr, uint8_t SubCmdByte, uint8_t *pData, uint32_t DataLen);

uint8_t I3C_CCC_GETXTIME(Gowin_I3C_TypeDef *I3Cx, uint8_t *pSlvAddr, TimingSupportInfo *pTimingSupportInfo, uint32_t SlvNum);
uint8_t I3C_CCC_GETXTIME_SingleSlave(Gowin_I3C_TypeDef *I3Cx, uint8_t SlvAddr, TimingSupportInfo *pTimingSupportInfo);

uint8_t I3C_CCC_RSTACT_Write(Gowin_I3C_TypeDef *I3Cx, uint8_t *pSlvAddr, uint8_t DefByte, uint8_t SlvNum);
uint8_t I3C_CCC_RSTACT_Read(Gowin_I3C_TypeDef *I3Cx, uint8_t *pSlvAddr, uint8_t DefByte, uint8_t *pRetData, uint32_t SlvNum);

uint8_t I3C_CCC_SETGRPA(Gowin_I3C_TypeDef *I3Cx, uint8_t *pSlvAddr, uint8_t GrpAddr, uint8_t SlvNum);
uint8_t I3C_CCC_RSTGRPA(Gowin_I3C_TypeDef *I3Cx, uint8_t *pSlvAddr, uint8_t SlvNum);

uint64_t get6ByteID(uint8_t *pUniqueIDBuf);

void PrintENTDAA_RdBk(I3C_ENTDAA_RdBk *pEntDaaRdBkBuf);




#ifdef __cplusplus
}
#endif

#endif

