/*
 * *****************************************************************************************
 *
 * 		Copyright (C) 2014-2019 Gowin Semiconductor Technology Co.,Ltd.
 * 		
 * @file			main.c
 * @author		Embedded Development Team
 * @version		V1.3.0
 * @date			2019-10-1 09:00:00
 * @brief			Main function.
 ******************************************************************************************
 */

/* Includes ------------------------------------------------------------------*/
#include "gw1ns4c.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "gowin_i3c.h"
#include "gowin_i3c_ccc.h"
#include "hw.h"


void i3c_test_setdasa_write_read(void);
void i3c_test_entdaa_multiple_slave_write_read(void);
void i3c_test_setaada_write_read(void);
void i3c_test_i2c_mode(void);
void i3c_test_setnewda_multiple_slave_write_read(void);
void i3c_test_entasx(void);
void i3c_test_mwl(void);
void i3c_test_mrl(void);
void i3c_test_enec_disec(void);
void i3c_test_getbcr(void);
void i3c_test_enttm(void);
void i3c_test_setxtime(void);
void i3c_test_rstact(void);
void i3c_test_group(void);
void i3c_test_ibi(void);
void i3c_test_ibi_manual_ack(void);
void i3c_ddr(void);
void i3c_ref_design(void);


int int_cnt = 0;
uint32_t mstatus_buff[64];

I3C_MSTS_RegDef msts_buf;
uint8_t  flag;

I3C_ENTDAA_RdBk EntDaaRdBkBuf[2];



int main(void)
{
	SystemInit();
	UartInit();
	GPIO0_Init();
	UART_SendString(UART0, "\r\n\r\n  ======I3C TEST BEGIN======  \r\n\r\n");

	gowin_i3c_init(I3C0);
	
	gowin_i3c_set_IBIRspType(I3C0, I3C_IBISPTYPE_AUTO_ACK);
	
	UART_SendString(UART0, "\r\n		Powre On 		\r\n");
	Delay(8333000);
	//Delay(8333000);
	
	// test 1: SETSADA, write slave reg , read slave reg
//	i3c_test_setdasa_write_read();

	// test 2: ENTDAA multiple slave
//	i3c_test_entdaa_multiple_slave_write_read();

	// test 3: SETAASA, write, read
//	i3c_test_setaada_write_read();
	
	// test 4: I2C Mode R/W
//	i3c_test_i2c_mode();
	
	// test 5: ENEC/DISEC
//	i3c_test_enec_disec();
	
	// test 6: GETBCR
//	i3c_test_getbcr();
	
	// test 7: SETNEWDA multiple slave
//	i3c_test_setnewda_multiple_slave_write_read();

	// test 8: ENTASx
//	i3c_test_entasx();

	// test 9: SETMWL/GETMWL
//	i3c_test_mwl();

	// test 10: SETMRL/GETMRL
//	i3c_test_mrl();

	// test 11: ENTTM
//	i3c_test_enttm();
	
	// test 12: SETXTIME
//	i3c_test_setxtime();
	
	// test 13: RSTACT
//	i3c_test_rstact();
	
	// test 14: group
//	i3c_test_group();

	// test 15: ibi and hj with manual ack
//	i3c_test_ibi_manual_ack();
	
	// test 16: ibi and hj with auto ack
//	i3c_test_ibi();

	// test 17: ddr test
//	i3c_ddr();	
	// test 18: Ref_design
	i3c_ref_design();

	UART_SendString(UART0, "\r\n\r\n  =======TEST  FINISH=======  \r\n\r\n");
	while(1);

}

#define RX_BYTE_NUM  16
#define TX_BYTE_NUM  4
#define I3C_DDR_CRC 0

void i3c_ref_design(void)
{
	//single slave ENTDAA
	uint32_t i;
	int nack;
	uint8_t wData = 0x55;
	uint8_t rData = 0;
	uint8_t RegAddr = 0x0c;
	uint8_t slaveNum = 1;
	volatile I3C_MCONTROL_RegDef MControlBuf;

	// ====================== ENTDAA
  uint8_t DynamicAddr = 0x21;
	UART_SendString(UART0, "	=== ENTDAA ===\r\n");
	nack = I3C_CCC_ENTDAA(I3C0, &DynamicAddr, slaveNum, EntDaaRdBkBuf);	
	printf("entdaa: %d\r\n", nack);
	PrintENTDAA_RdBk(&(EntDaaRdBkBuf[0]));
	printf("	entdaa finished...\r\n\r\n");
	// ====================== SDR Write & Read (Dynamic Address)
	printf("	===SDR Write & Read Data === \r\n");
	gowin_i3c_clear_sfifo(I3C0);	//clear send fifo
	gowin_i3c_clear_rfifo(I3C0);	//clear rev fifo
	
	printf("i3c write reg of slave ...\r\n");
	nack = I3C_Write_Slave_Reg(I3C0, I3C_COMTYPE_I3C, DynamicAddr,RegAddr , wData);
	if(nack) printf("Write nack\r\n");
	printf("i3c read reg of slave ...\r\n");
	nack = I3C_Read_Slave_Reg(I3C0, I3C_COMTYPE_I3C, DynamicAddr, 0x0c, &rData);
	if(nack) printf("Read nack\r\n");
	else printf("read back data: 0x%x\r\n",rData);
	if(wData == rData) {
		printf("\r\n	Slave Write & READ Success ...\r\n\r\n");
	}	
	
	// ======================= DDR Read 
	gowin_i3c_clear_sfifo(I3C0);	//clear send fifo
	gowin_i3c_clear_rfifo(I3C0);	//clear rev fifo
	
	uint8_t RxData1[64] = {0};
	uint8_t ddrRead = 0x80;
	printf("\r\n	=== I3C DDR Test === ...\r\n");
		
	nack = I3C_DDR_Read(I3C0, DynamicAddr, ddrRead, RxData1, RX_BYTE_NUM, I3C_DDR_CRC);
	I3C_ExitDDR(I3C0, DynamicAddr);
	printf("1 rx data: ");
	for(i = 0; i < RX_BYTE_NUM; i++) {
		printf("%02x ", RxData1[i]);
	}
	printf("\r\n");
	printf("nack: %d\r\n", nack);

}



void i3c_ddr(void)
{
	uint32_t i;
	uint8_t DynamicAddr = 0x34;
	uint8_t RxData1[64] = {0};
	uint8_t RxData2[64] = {0};
//	uint8_t TxData[16] = {0x11, 0x22, 0x33, 0x44};
	uint8_t Nack = 0;
	volatile I3C_MCONTROL_RegDef MControlBuf;
	
	
	I3C_CCC_ENTDAA(I3C0, &DynamicAddr, 1, EntDaaRdBkBuf);	

	
//	uint8_t Cmd;
	
	// ddr write test
//	for(Cmd = 0; Cmd < 0x80; Cmd++) {
//		Nack = I3C_DDR_Write(I3C0, DynamicAddr, Cmd, TxData, TX_BYTE_NUM, I3C_DDR_CRC);
//		I3C_ExitDDR(I3C0, DynamicAddr);
//		printf("Nack %02x: %d\r\n", Cmd, Nack);
//		I3C0->MDATACONTROL.SFIFOCLR = 1;
//		Delay(833300);
//	}
	
	// ddr read test
	Nack = I3C_DDR_Read(I3C0, DynamicAddr, 0x80, RxData1, RX_BYTE_NUM, I3C_DDR_CRC);
	I3C_ExitDDR(I3C0, DynamicAddr);
	printf("1 rx data: ");
	for(i = 0; i < RX_BYTE_NUM; i++) {
		printf("%02x ", RxData1[i]);
	}
	printf("\r\n");
	printf("Nack: %d\r\n", Nack);


	// ddr multi read test
	Nack = I3C_DDR_Read(I3C0, DynamicAddr, 0x80, RxData1, RX_BYTE_NUM, I3C_DDR_CRC);
	if(Nack) printf("nack\r\n");
	Nack = I3C_DDR_Read(I3C0, DynamicAddr, 0x80, RxData2, RX_BYTE_NUM, I3C_DDR_CRC);
	if(Nack) printf("nack\r\n");
	I3C_ExitDDR(I3C0, DynamicAddr);
	printf("1 rx data: ");
	for(i = 0; i < RX_BYTE_NUM; i++) {
		printf("%02x ", RxData1[i]);
	}
	printf("\r\n");
	printf("2 rx data: ");
	for(i = 0; i < RX_BYTE_NUM; i++) {
		printf("%02x ", RxData2[i]);
	}
	printf("\r\n");
	printf("Nack: %d\r\n", Nack);

}



void i3c_test_ibi_manual_ack(void)
{
	uint32_t i = 0;

	uint8_t StaticAddr  = 0x5c;
	uint8_t DynamicAddr = 0x2b;
	uint8_t Nack;
	uint8_t IBIRspData[4];
	uint8_t press_xl;
	uint8_t press_l;
	uint8_t press_h;
	uint32_t press_lsb;
	float press;
	
	uint8_t ibiPayloadSize = 3;
//	uint16_t mrl = 0x10;
	
	uint8_t temp_l;
	uint8_t temp_h;
	uint16_t temp_lsb;
	float temp;
	
	uint8_t ctrl_reg1, ctrl_reg2, ctrl_reg3, fifo_ctrl, fifo_wtm;
	
	uint8_t FifoStatus1;
	uint8_t FifoStatus2;
	uint8_t SensorStatus;
	uint8_t IntSource;
	
	SlaveEventsCmdByteDef SECbuf;
	SECbuf.ENHJ  = 1;
	SECbuf.ENINT = 1;
	SECbuf.ENMR  = 1;
	

	int_cnt = 0;
	
	
    // intitialize i3c slave (lps22hh), block it and do the initialization when HJ
	// NOTE: If testing HJ, the Int1 Pin should be connected to VCC, if not testing HJ, this Pin should be float
//	Nack = I3C_CCC_SETDASA(I3C0, &StaticAddr, &CCC_Data, 1);
//	if(Nack) {
//		printf("Incorrect Slave Address\r\n");
//	}
	
//	I3C_CCC_SETMRL(I3C0, &DynamicAddr, &mrl, &ibiPayloadSize, 1);

//	UART_SendString(UART0, "SETDASA\r\n");
//	
//	gowin_i3c_set_IBIRspType(I3C0, I3C_IBISPTYPE_MANUAL_ACK);
//	
//	
//	I3C0->MIBIFORMCFG.RegValue = 0x4000002b;
//	
//	Nack = I3C_Write_Slave_Reg(I3C0, I3C_COMTYPE_I3C, DynamicAddr, 0x10, 0x70);
//	Nack = I3C_Write_Slave_Reg(I3C0, I3C_COMTYPE_I3C, DynamicAddr, 0x11, 0x11);
//	Nack = I3C_Write_Slave_Reg(I3C0, I3C_COMTYPE_I3C, DynamicAddr, 0x12, 0x3c);
//	Nack = I3C_Write_Slave_Reg(I3C0, I3C_COMTYPE_I3C, DynamicAddr, 0x13, 0x01);
//	Nack = I3C_Write_Slave_Reg(I3C0, I3C_COMTYPE_I3C, DynamicAddr, 0x14, 0x01);


	
	while(1) {
		if(flag == 1) {
			flag = 0;
			if(msts_buf.SSTART) {
			}
			
			// slave hot join interrupt
			if(msts_buf.IBIRCV & (msts_buf.SRTYPE == 3)) {
				
				Nack = I3C_CCC_SETDASA(I3C0, &StaticAddr, &DynamicAddr, 1);
				if(Nack) {
					printf("Incorrect Slave Address\r\n");
				}

				UART_SendString(UART0, "SETDASA after HJ\r\n");
				
				gowin_i3c_set_IBIRspType(I3C0, I3C_IBISPTYPE_MANUAL_ACK);
				
//				I3C_CCC_SETMRL(I3C0, &DynamicAddr, &mrl, &ibiPayloadSize, 1);
				
				I3C0->MIBIFORMCFG.RegValue = 0x4000002b;
				
				Nack = I3C_Write_Slave_Reg(I3C0, I3C_COMTYPE_I3C, DynamicAddr, 0x10, 0x70);
				Nack = I3C_Write_Slave_Reg(I3C0, I3C_COMTYPE_I3C, DynamicAddr, 0x11, 0x11);
				Nack = I3C_Write_Slave_Reg(I3C0, I3C_COMTYPE_I3C, DynamicAddr, 0x12, 0x3c);
				Nack = I3C_Write_Slave_Reg(I3C0, I3C_COMTYPE_I3C, DynamicAddr, 0x13, 0x01);
				Nack = I3C_Write_Slave_Reg(I3C0, I3C_COMTYPE_I3C, DynamicAddr, 0x14, 0x01);							
			}
			
			// slave ibi interrupt
			if(msts_buf.IBIRCV & (msts_buf.SRTYPE == 1)) {
				// manual ack
//				I3C0->MCONTROL.RegValue = 0x0000fc83;				
//				I3C_Wait_MControl_Finish(I3C0);
				
				I3C_IBI_Manual_Ack(I3C0, I3C_MANUAL_ACK_TYPE_ACK_WITH_MBYTE);

				I3C_ReadRxData(I3C0, IBIRspData, ibiPayloadSize + 1);				
				I3C_SendStop(I3C0);
				
				
				
				I3C_CCC_Broadcast(I3C0, I3C_CCC_CODE_BROADCAST_DISEC, &(SECbuf.ByteValue), NULL, 0); 
				I3C_Read_Slave_Reg(I3C0, I3C_COMTYPE_I3C, DynamicAddr, 0x78, &press_xl);
				I3C_Read_Slave_Reg(I3C0, I3C_COMTYPE_I3C, DynamicAddr, 0x79, &press_l);
				I3C_Read_Slave_Reg(I3C0, I3C_COMTYPE_I3C, DynamicAddr, 0x7a, &press_h);
				I3C_Read_Slave_Reg(I3C0, I3C_COMTYPE_I3C, DynamicAddr, 0x7b, &temp_l);
				I3C_Read_Slave_Reg(I3C0, I3C_COMTYPE_I3C, DynamicAddr, 0x7c, &temp_h);
				
				I3C_Read_Slave_Reg(I3C0, I3C_COMTYPE_I3C, DynamicAddr, 0x24, &IntSource);
				I3C_Read_Slave_Reg(I3C0, I3C_COMTYPE_I3C, DynamicAddr, 0x25, &FifoStatus1);
				I3C_Read_Slave_Reg(I3C0, I3C_COMTYPE_I3C, DynamicAddr, 0x26, &FifoStatus2);
				I3C_Read_Slave_Reg(I3C0, I3C_COMTYPE_I3C, DynamicAddr, 0x27, &SensorStatus);

				I3C_Read_Slave_Reg(I3C0, I3C_COMTYPE_I3C, DynamicAddr, 0x10, &ctrl_reg1);
				I3C_Read_Slave_Reg(I3C0, I3C_COMTYPE_I3C, DynamicAddr, 0x11, &ctrl_reg2);
				I3C_Read_Slave_Reg(I3C0, I3C_COMTYPE_I3C, DynamicAddr, 0x12, &ctrl_reg3);
				I3C_Read_Slave_Reg(I3C0, I3C_COMTYPE_I3C, DynamicAddr, 0x13, &fifo_ctrl);
				I3C_Read_Slave_Reg(I3C0, I3C_COMTYPE_I3C, DynamicAddr, 0x14, &fifo_wtm);
				
				press_lsb = press_xl + (press_l << 8) + (press_h << 16);
				temp_lsb = temp_l + (temp_h << 8);
				press = ((float)(press_lsb)) / 4096.0;
				temp = ((float)(temp_lsb)) / 100.0;				

				printf("press: %.2f hPA\r\n", press);
				printf("temp:  %.2f degree\r\n", temp);


				for(i = 0; i < ibiPayloadSize + 1; i++) {
					printf("ibi rsp: %02x\r\n", IBIRspData[i]);
				}
				
				printf("ctrl_reg1:      %02x\r\n", ctrl_reg1);
				printf("ctrl_reg2:      %02x\r\n", ctrl_reg2);
				printf("ctrl_reg3:      %02x\r\n", ctrl_reg3);
				printf("fifo_ctrl:      %02x\r\n", fifo_ctrl);
				printf("fifo_wtm:       %02x\r\n", fifo_wtm);
				printf("int source:     %02x\r\n", IntSource);
				printf("fifo status1:   %02x\r\n", FifoStatus1);
				printf("fifo status2:   %02x\r\n", FifoStatus2);
				printf("sensor status2: %02x\r\n", SensorStatus);

//				if(int_cnt < 600) {
//					I3C_CCC_Broadcast(I3C0, I3C_CCC_CODE_BROADCAST_ENEC, &(SECbuf.ByteValue), NULL, 0); 
//				}
				I3C_CCC_Broadcast(I3C0, I3C_CCC_CODE_BROADCAST_ENEC, &(SECbuf.ByteValue), NULL, 0); 
				
				
//				Nack = I3C_Write_Slave_Reg(I3C0, I3C_COMTYPE_I3C, DynamicAddr, 0x10, 0x70);
//				Nack = I3C_Write_Slave_Reg(I3C0, I3C_COMTYPE_I3C, DynamicAddr, 0x11, 0x11);
//				Nack = I3C_Write_Slave_Reg(I3C0, I3C_COMTYPE_I3C, DynamicAddr, 0x12, 0x3c);
//				Nack = I3C_Write_Slave_Reg(I3C0, I3C_COMTYPE_I3C, DynamicAddr, 0x13, 0x01);
//				Nack = I3C_Write_Slave_Reg(I3C0, I3C_COMTYPE_I3C, DynamicAddr, 0x14, 0x01);
//				
//				I3C0->MIBIFORMCFG.RegValue = 0x4000002b;									
			}
		}
	}
	
//	for(i = 0; i < int_cnt; i++) {
//		printf("mstatus %02d: %08x\r\n", i, mstatus_buff[i]);
//	}
	


}



void i3c_test_ibi(void)
{
	uint32_t i = 0;

	uint8_t StaticAddr  = 0x5c;
	uint8_t DynamicAddr = 0x2b;
	uint8_t Nack;
	uint8_t IBIRspData[4];
	uint8_t press_xl;
	uint8_t press_l;
	uint8_t press_h;
	uint32_t press_lsb;
	float press;
	
	uint8_t ibiPayloadSize = 3;
//	uint16_t mrl = 0x10;
	
	uint8_t temp_l;
	uint8_t temp_h;
	uint16_t temp_lsb;
	float temp;
	
	uint8_t ctrl_reg1, ctrl_reg2, ctrl_reg3, fifo_ctrl, fifo_wtm;
	
	uint8_t FifoStatus1;
	uint8_t FifoStatus2;
	uint8_t SensorStatus;
	uint8_t IntSource;
	
	SlaveEventsCmdByteDef SECbuf;
	SECbuf.ENHJ  = 1;
	SECbuf.ENINT = 1;
	SECbuf.ENMR  = 1;
	

	int_cnt = 0;
	
	
    // intitialize i3c slave (lps22hh), block it and do the initialization when HJ
	// NOTE: If testing HJ, the Int1 Pin should be connected to VCC, if not testing HJ, this Pin should be float
//	Nack = I3C_CCC_SETDASA(I3C0, &StaticAddr, &CCC_Data, 1);
//	if(Nack) {
//		printf("Incorrect Slave Address\r\n");
//	}
//	
////	I3C_CCC_SETMRL(I3C0, &DynamicAddr, &mrl, &ibiPayloadSize, 1);

//	UART_SendString(UART0, "SETDASA\r\n");
//	
//	gowin_i3c_set_IBIRspType(I3C0, I3C_IBISPTYPE_MANUAL_ACK);
//	
//	
//	I3C0->MIBIFORMCFG.RegValue = 0x4000002b;
//	
//	Nack = I3C_Write_Slave_Reg(I3C0, I3C_COMTYPE_I3C, DynamicAddr, 0x10, 0x70);
//	Nack = I3C_Write_Slave_Reg(I3C0, I3C_COMTYPE_I3C, DynamicAddr, 0x11, 0x11);
//	Nack = I3C_Write_Slave_Reg(I3C0, I3C_COMTYPE_I3C, DynamicAddr, 0x12, 0x3c);
//	Nack = I3C_Write_Slave_Reg(I3C0, I3C_COMTYPE_I3C, DynamicAddr, 0x13, 0x01);
//	Nack = I3C_Write_Slave_Reg(I3C0, I3C_COMTYPE_I3C, DynamicAddr, 0x14, 0x01);


	
	while(1) {
		if(flag == 1) {
			flag = 0;
			if(msts_buf.SSTART) {
			}
			
			// slave hot join interrupt
			if(msts_buf.IBIRCV & (msts_buf.SRTYPE == 3)) {
				
				Nack = I3C_CCC_SETDASA(I3C0, &StaticAddr, &DynamicAddr, 1);
				if(Nack) {
					printf("Incorrect Slave Address\r\n");
				}

				UART_SendString(UART0, "SETDASA after HJ\r\n");
				
				gowin_i3c_set_IBIRspType(I3C0, I3C_IBISPTYPE_AUTO_ACK);
				
//				I3C_CCC_SETMRL(I3C0, &DynamicAddr, &mrl, &ibiPayloadSize, 1);
				
				I3C0->MIBIFORMCFG.RegValue = 0x4000002b;
				
				Nack = I3C_Write_Slave_Reg(I3C0, I3C_COMTYPE_I3C, DynamicAddr, 0x10, 0x70);
				Nack = I3C_Write_Slave_Reg(I3C0, I3C_COMTYPE_I3C, DynamicAddr, 0x11, 0x11);
				Nack = I3C_Write_Slave_Reg(I3C0, I3C_COMTYPE_I3C, DynamicAddr, 0x12, 0x3c);
				Nack = I3C_Write_Slave_Reg(I3C0, I3C_COMTYPE_I3C, DynamicAddr, 0x13, 0x01);
				Nack = I3C_Write_Slave_Reg(I3C0, I3C_COMTYPE_I3C, DynamicAddr, 0x14, 0x01);							
			}
			
			// slave ibi interrupt
			if(msts_buf.IBIRCV & (msts_buf.SRTYPE == 1)) {
				I3C_ReadRxData(I3C0, IBIRspData, ibiPayloadSize + 1);
				I3C_SendStop(I3C0);
				I3C_CCC_Broadcast(I3C0, I3C_CCC_CODE_BROADCAST_DISEC, &(SECbuf.ByteValue), NULL, 0); 
				I3C_Read_Slave_Reg(I3C0, I3C_COMTYPE_I3C, DynamicAddr, 0x78, &press_xl);
				I3C_Read_Slave_Reg(I3C0, I3C_COMTYPE_I3C, DynamicAddr, 0x79, &press_l);
				I3C_Read_Slave_Reg(I3C0, I3C_COMTYPE_I3C, DynamicAddr, 0x7a, &press_h);
				I3C_Read_Slave_Reg(I3C0, I3C_COMTYPE_I3C, DynamicAddr, 0x7b, &temp_l);
				I3C_Read_Slave_Reg(I3C0, I3C_COMTYPE_I3C, DynamicAddr, 0x7c, &temp_h);
				
				I3C_Read_Slave_Reg(I3C0, I3C_COMTYPE_I3C, DynamicAddr, 0x24, &IntSource);
				I3C_Read_Slave_Reg(I3C0, I3C_COMTYPE_I3C, DynamicAddr, 0x25, &FifoStatus1);
				I3C_Read_Slave_Reg(I3C0, I3C_COMTYPE_I3C, DynamicAddr, 0x26, &FifoStatus2);
				I3C_Read_Slave_Reg(I3C0, I3C_COMTYPE_I3C, DynamicAddr, 0x27, &SensorStatus);

				I3C_Read_Slave_Reg(I3C0, I3C_COMTYPE_I3C, DynamicAddr, 0x10, &ctrl_reg1);
				I3C_Read_Slave_Reg(I3C0, I3C_COMTYPE_I3C, DynamicAddr, 0x11, &ctrl_reg2);
				I3C_Read_Slave_Reg(I3C0, I3C_COMTYPE_I3C, DynamicAddr, 0x12, &ctrl_reg3);
				I3C_Read_Slave_Reg(I3C0, I3C_COMTYPE_I3C, DynamicAddr, 0x13, &fifo_ctrl);
				I3C_Read_Slave_Reg(I3C0, I3C_COMTYPE_I3C, DynamicAddr, 0x14, &fifo_wtm);
				
				press_lsb = press_xl + (press_l << 8) + (press_h << 16);
				temp_lsb = temp_l + (temp_h << 8);
				press = ((float)(press_lsb)) / 4096.0;
				temp = ((float)(temp_lsb)) / 100.0;				

				printf("press: %.2f hPA\r\n", press);
				printf("temp:  %.2f degree\r\n", temp);


				for(i = 0; i < ibiPayloadSize + 1; i++) {
					printf("ibi rsp: %02x\r\n", IBIRspData[i]);
				}
				
				printf("ctrl_reg1:      %02x\r\n", ctrl_reg1);
				printf("ctrl_reg2:      %02x\r\n", ctrl_reg2);
				printf("ctrl_reg3:      %02x\r\n", ctrl_reg3);
				printf("fifo_ctrl:      %02x\r\n", fifo_ctrl);
				printf("fifo_wtm:       %02x\r\n", fifo_wtm);
				printf("int source:     %02x\r\n", IntSource);
				printf("fifo status1:   %02x\r\n", FifoStatus1);
				printf("fifo status2:   %02x\r\n", FifoStatus2);
				printf("sensor status2: %02x\r\n", SensorStatus);

//				if(int_cnt < 600) {
//					I3C_CCC_Broadcast(I3C0, I3C_CCC_CODE_BROADCAST_ENEC, &(SECbuf.ByteValue), NULL, 0); 
//				}
				I3C_CCC_Broadcast(I3C0, I3C_CCC_CODE_BROADCAST_ENEC, &(SECbuf.ByteValue), NULL, 0); 
				
				
//				Nack = I3C_Write_Slave_Reg(I3C0, I3C_COMTYPE_I3C, DynamicAddr, 0x10, 0x70);
//				Nack = I3C_Write_Slave_Reg(I3C0, I3C_COMTYPE_I3C, DynamicAddr, 0x11, 0x11);
//				Nack = I3C_Write_Slave_Reg(I3C0, I3C_COMTYPE_I3C, DynamicAddr, 0x12, 0x3c);
//				Nack = I3C_Write_Slave_Reg(I3C0, I3C_COMTYPE_I3C, DynamicAddr, 0x13, 0x01);
//				Nack = I3C_Write_Slave_Reg(I3C0, I3C_COMTYPE_I3C, DynamicAddr, 0x14, 0x01);
//				
//				I3C0->MIBIFORMCFG.RegValue = 0x4000002b;									
			}
		}
	}
	
//	for(i = 0; i < int_cnt; i++) {
//		printf("mstatus %02d: %08x\r\n", i, mstatus_buff[i]);
//	}
	


}


void i3c_test_group(void)
{
	uint8_t DynamicAddr[2] = {0x21, 0x6b};
	uint8_t GrpAddr = 0x5a;
	I3C_CCC_ENTDAA(I3C0, DynamicAddr, 2, EntDaaRdBkBuf);
	
	DynamicAddr[0] = 0x6b; DynamicAddr[1] = 0x21;
	I3C_CCC_SETGRPA(I3C0, DynamicAddr, GrpAddr, 2);
	I3C_CCC_RSTGRPA(I3C0, DynamicAddr, 2);
	
}


void i3c_test_rstact(void)
{
	uint8_t DefByte = 0x01;
	uint8_t DynamicAddr[2] = {0x21, 0x6b};
	uint8_t RetData[64];
	I3C_CCC_ENTDAA(I3C0, DynamicAddr, 2, EntDaaRdBkBuf);	
	I3C_CCC_Broadcast(I3C0, I3C_CCC_CODE_BROADCAST_RSTACT, &DefByte, NULL, 0);
	
	DynamicAddr[0] = 0x6b;	
	I3C_CCC_RSTACT_Write(I3C0, DynamicAddr, DefByte, 2);
	
	I3C_CCC_RSTACT_Read(I3C0, DynamicAddr, DefByte, RetData, 1);
}


void i3c_test_setxtime(void)
{
	uint8_t DynamicAddr[2] = {0x21, 0x6b};
	uint8_t FuncByte = 0x7f;
	
	uint8_t FuncBytes[2] = {0xbf, 0x3f};
	uint8_t Data_bf = 0x11;
	uint8_t Data_3f[2] = {0x22, 0x33};
	uint32_t DataLen[2] = {1, 2};
	uint8_t *Data[2];
	
	Data[0] = &Data_bf; 
	Data[1] = Data_3f; 
	
	TimingSupportInfo TSInfo[2];
	
	
	I3C_CCC_ENTDAA(I3C0, DynamicAddr, 2, EntDaaRdBkBuf);	
	
	I3C_CCC_GETXTIME_SingleSlave(I3C0, DynamicAddr[0], &TSInfo[0]);
	printf("SupportedModes: %02x, State: %02x, Frequency: %02x, Inaccuracy: %02x\r\n", TSInfo[0].SupportedModes, TSInfo[0].State, TSInfo[0].Frequency, TSInfo[0].Inaccuracy);
	
	I3C_CCC_Broadcast(I3C0, I3C_CCC_CODE_BROADCAST_SETXTIME, &FuncByte, NULL, 0);
	I3C_CCC_Broadcast(I3C0, I3C_CCC_CODE_BROADCAST_SETXTIME, &FuncBytes[1], Data[1], 2);
	I3C_CCC_Broadcast(I3C0, I3C_CCC_CODE_BROADCAST_SETXTIME, &FuncBytes[1], Data[1], 2);


	
	I3C_CCC_SETXTIME(I3C0, DynamicAddr, FuncBytes, Data, DataLen, 2);

	I3C_CCC_GETXTIME(I3C0, DynamicAddr, TSInfo, 2);
	printf("SupportedModes0: %02x, State0: %02x, Frequency0: %02x, Inaccuracy0: %02x\r\n", TSInfo[0].SupportedModes, TSInfo[0].State, TSInfo[0].Frequency, TSInfo[0].Inaccuracy);	
	printf("SupportedModes1: %02x, State1: %02x, Frequency1: %02x, Inaccuracy1: %02x\r\n", TSInfo[0].SupportedModes, TSInfo[1].State, TSInfo[1].Frequency, TSInfo[1].Inaccuracy);	

	I3C_CCC_SETXTIME_SingleSlave(I3C0, DynamicAddr[0], FuncBytes[1], Data[1], DataLen[1]);
	I3C_CCC_GETXTIME_SingleSlave(I3C0, DynamicAddr[0], &TSInfo[0]);
	printf("SupportedModes: %02x, State: %02x, Frequency: %02x, Inaccuracy: %02x\r\n", TSInfo[0].SupportedModes, TSInfo[0].State, TSInfo[0].Frequency, TSInfo[0].Inaccuracy);

	
	
}


void i3c_test_enttm(void)
{
	uint8_t DynamicAddr[2] = {0x6b, 0x21};
	uint8_t TestModeByte = 0x01;
	
	I3C_CCC_ENTDAA(I3C0, DynamicAddr, 2, EntDaaRdBkBuf);	
	I3C_CCC_Broadcast(I3C0, I3C_CCC_CODE_BROADCAST_ENTTM, NULL, &TestModeByte, 1);
	
	TestModeByte = 0x00;
	I3C_CCC_Broadcast(I3C0, I3C_CCC_CODE_BROADCAST_ENTTM, NULL, &TestModeByte, 1);
	
	
}



void i3c_test_mwl(void)
{
	uint8_t Nack;
	uint8_t DynamicAddr[2] = {0x6b, 0x21};
	uint8_t Mwl[2] = {0x12, 0x34};
	
	uint16_t DirMwl[2] = {0x1111, 0x2222};
	uint16_t DirMwl_Rdbk[2];
	
	Nack = 0;
	
	UART_SendString(UART0, "SETMWL\r\n");
	I3C_CCC_ENTDAA(I3C0, DynamicAddr, 2, EntDaaRdBkBuf);
	
	
	Nack = I3C_CCC_Broadcast(I3C0, I3C_CCC_CODE_BROADCAST_SETMWL, NULL, Mwl, 2);
	if(Nack) printf("nack\r\n");
	Nack = I3C_CCC_GETMWL(I3C0, DynamicAddr, DirMwl_Rdbk, 2);
	if(Nack) printf("nack\r\n");
	printf("MWL0: %04x, MWL1: %04x\r\n", DirMwl_Rdbk[0], DirMwl_Rdbk[1]);
	
	Nack = I3C_CCC_SETMWL(I3C0, DynamicAddr, DirMwl, 2);
	if(Nack) printf("nack\r\n");
	Nack = I3C_CCC_GETMWL(I3C0, DynamicAddr, DirMwl_Rdbk, 2);
	if(Nack) printf("nack\r\n");
	printf("MWL0: %04x, MWL1: %04x\r\n", DirMwl_Rdbk[0], DirMwl_Rdbk[1]);

}


void i3c_test_mrl(void)
{
	uint8_t DynamicAddr[2] = {0x6b, 0x21};
	uint8_t Mrl[2] = {0x12, 0x34};
	
	uint16_t DirMrl[2] = {0x1111, 0x2222};
	uint16_t DirMrl_Rdbk[2];
	uint8_t IBIpayloadsize[2];
	uint8_t IBIpayloadsize_rdbk[2];
	
	UART_SendString(UART0, "SETMRL\r\n");
	I3C_CCC_ENTDAA(I3C0, DynamicAddr, 2, EntDaaRdBkBuf);	
	
	I3C_CCC_Broadcast(I3C0, I3C_CCC_CODE_BROADCAST_SETMRL, NULL, Mrl, 2);
	I3C_CCC_GETMRL(I3C0, DynamicAddr, DirMrl_Rdbk, IBIpayloadsize, 1);
	printf("MRL0: %04x, MRL1: %04x\r\n", DirMrl_Rdbk[0], DirMrl_Rdbk[1]);
	printf("ibi payload size: %d, %d\r\n", IBIpayloadsize[0], IBIpayloadsize[1]);
	
	IBIpayloadsize[0] = 2; IBIpayloadsize[0] = 2;
	I3C_CCC_SETMRL(I3C0, DynamicAddr, DirMrl, IBIpayloadsize, 2);
	I3C_CCC_GETMRL(I3C0, DynamicAddr, DirMrl_Rdbk, IBIpayloadsize_rdbk, 1);
	printf("MRL0: %04x, MRL1: %04x\r\n", DirMrl_Rdbk[0], DirMrl_Rdbk[1]);
	printf("ibi payload size: %d, %d\r\n", IBIpayloadsize_rdbk[0], IBIpayloadsize_rdbk[1]);
	
	
}



void i3c_test_entasx(void)
{
	uint8_t DynamicAddr[2] = {0x6b, 0x21};
	

	UART_SendString(UART0, "ENTASX\r\n");
	I3C_CCC_ENTDAA(I3C0, DynamicAddr, 2, EntDaaRdBkBuf);	
	
	I3C_CCC_Broadcast(I3C0, I3C_CCC_CODE_BROADCAST_ENTAS1, NULL, NULL, 0);
	I3C_CCC_Broadcast(I3C0, I3C_CCC_CODE_BROADCAST_ENTAS2, NULL, NULL, 0);
	I3C_CCC_Broadcast(I3C0, I3C_CCC_CODE_BROADCAST_ENTAS3, NULL, NULL, 0);
	
	
	I3C_CCC_ENTASx_Direct(I3C0, I3C_CCC_CODE_DIRECT_ENTAS1, DynamicAddr, 2);
	I3C_CCC_ENTASx_Direct(I3C0, I3C_CCC_CODE_DIRECT_ENTAS2, DynamicAddr, 2);
	I3C_CCC_ENTASx_Direct(I3C0, I3C_CCC_CODE_DIRECT_ENTAS3, DynamicAddr, 2);
	return;
}



void i3c_test_setdasa_write_read(void)
{
	uint8_t rData = 0;
	uint8_t wData = 0x44;
//	uint8_t StaticAddr  = 0x11; //0x5c; wrong address
	uint8_t StaticAddr  = 0x68; //0x5c;
	uint8_t DynamicAddr = StaticAddr+1;
	uint8_t Nack;
	
	UART_SendString(UART0, "SETDASA\r\n");
		

	Nack = I3C_CCC_SETDASA(I3C0, &StaticAddr, &DynamicAddr, 1);
	if(Nack) {
		printf("Incorrect Slave Address\r\n");
	}
	
	printf("i3c write reg of slave ...\r\n");
	Nack = I3C_Write_Slave_Reg(I3C0, I3C_COMTYPE_I3C, DynamicAddr, 0x0c, wData);
	if(Nack) {
		printf("Incorrect Slave Address\r\n");
	}
	printf("i3c read reg of slave ...\r\n");
    Nack = I3C_Read_Slave_Reg(I3C0, I3C_COMTYPE_I3C, DynamicAddr, 0x0c, &rData);	
	if(Nack) {
		printf("Incorrect Slave Address\r\n");
	}
	printf("read back data: 0x%x\r\n",rData);	
	
	if(wData == rData) {
		printf("Test Success...\r\n");
	}		
	
}


void i3c_test_entdaa_multiple_slave_write_read(void)
{
	int i;
	int nack;
	uint8_t rData = 0;
	uint8_t wData = 0x55;
	
	uint8_t DynamicAddr[2] = {0x6b, 0x21};

	UART_SendString(UART0, "ENTDAA\r\n");
	nack = I3C_CCC_ENTDAA(I3C0, DynamicAddr, 2, EntDaaRdBkBuf);	
	printf("entdaa: %d\r\n", nack);
	for(i = 0; i < 2; i++) {
		PrintENTDAA_RdBk(&(EntDaaRdBkBuf[i]));
	}	
	printf("entdaa finished...\r\n\r\n");
	
//	I3C0->MCFG.PPHIGH          = 3;
	
	printf("i3c write reg of slave 1...\r\n");
	nack = I3C_Write_Slave_Reg(I3C0, I3C_COMTYPE_I3C, DynamicAddr[0], 0x0c, wData);
	if(nack) printf("nack\r\n");
	printf("i3c read reg of slave 1...\r\n");
	nack = I3C_Read_Slave_Reg(I3C0, I3C_COMTYPE_I3C, DynamicAddr[0], 0x0c, &rData);
	if(nack) printf("nack\r\n");
	printf("read back data: 0x%x\r\n",rData);
	if(wData == rData) {
		printf("Slave 1 Success...\r\n\r\n");
	}

	rData = 0;
	wData = 0xaa;
	printf("i3c write reg of slave 2...\r\n");
	nack = I3C_Write_Slave_Reg(I3C0, I3C_COMTYPE_I3C, DynamicAddr[1], 0x15, wData);
	if(nack) printf("nack\r\n");
	printf("i3c read reg of slave 2...\r\n");
	nack = I3C_Read_Slave_Reg(I3C0, I3C_COMTYPE_I3C, DynamicAddr[1], 0x15, &rData);
	if(nack) printf("nack\r\n");
	printf("read back data: 0x%x\r\n",rData);
	if(wData == rData) {
		printf("Slave 2 Success...\r\n\r\n");
	}
	
	uint8_t NewDA[2] = {0x77, 0x66};
	
	nack = I3C_CCC_SETNEWDA(I3C0, DynamicAddr, NewDA, 2);
	
	printf("i3c write reg of slave 1...\r\n");
	nack = I3C_Write_Slave_Reg(I3C0, I3C_COMTYPE_I3C, NewDA[0], 0x0c, wData);
	if(nack) printf("nack\r\n");
	printf("i3c read reg of slave 1...\r\n");
	nack = I3C_Read_Slave_Reg(I3C0, I3C_COMTYPE_I3C, NewDA[0], 0x0c, &rData);
	if(nack) printf("nack\r\n");
	printf("read back data: 0x%x\r\n",rData);
	if(wData == rData) {
		printf("Slave 1 Success...\r\n\r\n");
	}

	rData = 0;
	wData = 0xaa;
	printf("i3c write reg of slave 2...\r\n");
	nack = I3C_Write_Slave_Reg(I3C0, I3C_COMTYPE_I3C, NewDA[1], 0x15, wData);
	if(nack) printf("nack\r\n");
	printf("i3c read reg of slave 2...\r\n");
	nack = I3C_Read_Slave_Reg(I3C0, I3C_COMTYPE_I3C, NewDA[1], 0x15, &rData);
	if(nack) printf("nack\r\n");
	printf("read back data: 0x%x\r\n",rData);
	if(wData == rData) {
		printf("Slave 2 Success...\r\n\r\n");
	}
	
	
}


void i3c_test_setnewda_multiple_slave_write_read(void)
{
	int i;
	int nack;
	uint8_t rData = 0;
	uint8_t wData = 0x55;
	
	uint8_t DynamicAddr[2] = {0x6b, 0x21};

	UART_SendString(UART0, "ENTDAA\r\n");
	nack = I3C_CCC_ENTDAA(I3C0, DynamicAddr, 2, EntDaaRdBkBuf);	
	printf("entdaa: %d\r\n", nack);
	for(i = 0; i < 2; i++) {
		PrintENTDAA_RdBk(&(EntDaaRdBkBuf[i]));
	}	
	printf("entdaa finished...\r\n\r\n");
	

	
	uint8_t NewDA[2] = {0x77, 0x66};
	
	printf("\r\nSETNEWDA\r\n");
	nack = I3C_CCC_SETNEWDA(I3C0, DynamicAddr, NewDA, 2);
	
	printf("i3c write reg of slave 1...\r\n");
	nack = I3C_Write_Slave_Reg(I3C0, I3C_COMTYPE_I3C, NewDA[0], 0x0c, wData);
	if(nack) printf("nack\r\n");
	printf("i3c read reg of slave 1...\r\n");
	nack = I3C_Read_Slave_Reg(I3C0, I3C_COMTYPE_I3C, NewDA[0], 0x0c, &rData);
	if(nack) printf("nack\r\n");
	printf("read back data: 0x%x\r\n",rData);
	if(wData == rData) {
		printf("Slave 1 Success...\r\n\r\n");
	}

	rData = 0;
	wData = 0xaa;
	printf("i3c write reg of slave 2...\r\n");
	nack = I3C_Write_Slave_Reg(I3C0, I3C_COMTYPE_I3C, NewDA[1], 0x0b, wData);
	if(nack) printf("nack\r\n");
	printf("i3c read reg of slave 2...\r\n");
	nack = I3C_Read_Slave_Reg(I3C0, I3C_COMTYPE_I3C, NewDA[1], 0x0b, &rData);
	if(nack) printf("nack\r\n");
	printf("read back data: 0x%x\r\n",rData);
	if(wData == rData) {
		printf("Slave 2 Success...\r\n\r\n");
	}
	
	
}




void i3c_test_setaada_write_read(void)
{
	uint8_t StaticAddr  = 0x68;
	uint8_t rData = 0;
	uint8_t wData = 0x55;
	
	UART_SendString(UART0, "SETAASA\r\n");
	I3C_CCC_Broadcast(I3C0, I3C_CCC_CODE_BROADCAST_SETAASA, NULL, NULL, 0);
	printf("i3c write reg of slave ...\r\n");
	I3C_Write_Slave_Reg(I3C0, I3C_COMTYPE_I3C, StaticAddr, 0x0c, wData);
	printf("i3c read reg of slave ...\r\n");
    I3C_Read_Slave_Reg(I3C0, I3C_COMTYPE_I3C, StaticAddr, 0x0c, &rData);	
	printf("read back data: 0x%x\r\n",rData);
	if(wData == rData) {
		printf("Test Success...\r\n");
	}
}

void i3c_test_i2c_mode(void)
{
	uint8_t Ack;
	uint8_t StaticAddr  = 0x68;
//	uint8_t StaticAddr  = 0x11; // incorrect address
	uint8_t rData = 0;
	uint8_t wData = 0xcc;
	
	UART_SendString(UART0, "I2C Mode R/W\r\n");
	printf("i3c write reg of slave in i2c mode ...\r\n");
	Ack = I3C_Write_Slave_Reg(I3C0, I3C_COMTYPE_I2C, StaticAddr, 0x0b, wData);
	if(Ack) {
		printf("Incorrect Slave Address\r\n");
		return;
	}
	printf("i3c read reg of slave in i2c mode ...\r\n");
    Ack = I3C_Read_Slave_Reg(I3C0, I3C_COMTYPE_I2C, StaticAddr, 0x0b, &rData);
	if(Ack) {
		printf("Incorrect Slave Address\r\n");
		return;
	}
	printf("read back data: 0x%x\r\n",rData);	
	if(wData == rData) {
		printf("Test Success...\r\n");
	}
}


void i3c_test_enec_disec(void)
{
	uint8_t SlvEvtByte[2] = {0x0b, 0x0b};
	uint8_t DynamicAddr[2] = {0x6b, 0x21};
	
	I3C_CCC_ENTDAA(I3C0, DynamicAddr, 2, EntDaaRdBkBuf);
	I3C_CCC_ENEC_DISEC_Direct(I3C0, I3C_CCC_CODE_DIRECT_ENEC, DynamicAddr, SlvEvtByte, 2);

	
	SlaveEventsCmdByteDef SECbuf;
	SECbuf.ENHJ  = 1;
	SECbuf.ENINT = 1;
	SECbuf.ENMR  = 1;
	
	I3C_CCC_Broadcast(I3C0, I3C_CCC_CODE_BROADCAST_DISEC, &(SECbuf.ByteValue), NULL, 0); 
	
}


void i3c_test_getbcr(void)
{
	int i;
	uint8_t DynamicAddr[2] = {0x21, 0x6b};
	
	I3C_CCC_ENTDAA(I3C0, DynamicAddr, 2, EntDaaRdBkBuf);
	for(i = 0; i < 2; i++) {
		PrintENTDAA_RdBk(&(EntDaaRdBkBuf[i]));
	}
	
	
	uint8_t BCRByte[2];
	I3C_CCC_GETBCR(I3C0, DynamicAddr, BCRByte, 2);	
	for(i = 0; i < 2; i++) {
		printf("BCR%d : %02x\r\n", i, BCRByte[i]);
	}
	
	uint8_t DCRByte[2];
	I3C_CCC_GETDCR(I3C0, DynamicAddr, DCRByte, 2);	
	for(i = 0; i < 2; i++) {
		printf("DCR%d : %02x\r\n", i, DCRByte[i]);
	}

	uint64_t PID[2];	
	I3C_CCC_GETPID(I3C0, DynamicAddr, PID, 2);
	for(i = 0; i < 2; i++) {
		printf("PID: %d : %012llx\r\n", i, PID[i]);
	}

	uint16_t Status[2];
	I3C_CCC_GETSTATUS(I3C0, DynamicAddr, Status, 2);
	for(i = 0; i < 2; i++) {
		printf("Status %d: %04x\r\n", i, Status[i]);
	}



	
	return;
}








