`timescale 1ns/1ps

module ts(
    inout                       SCL,
    inout                       SDA
);

reg                         sda_oen = 1;
reg                         sda_out = 1;
reg         [9:0]           cnt = 0;

wire                        scl_line;
wire                        sda_line;
wire                        scl_falling_edge;
wire                        scl_rising_edge;
reg                         scl_line_reg_d1;
reg                         scl_line_reg_d2;
reg                         scl_line_reg_d3;
reg                         sclk = 0;

assign scl_falling_edge = !scl_line_reg_d2 && scl_line_reg_d3;
assign scl_rising_edge = scl_line_reg_d2 && !scl_line_reg_d3;

always #5 sclk = !sclk;

assign scl_line = SCL;

assign sda_line = SDA;
assign SDA = (!sda_oen) ? sda_out : 1'bz;

initial begin
    //7E + W ACK
    wait(cnt == 9);
    @(negedge scl_falling_edge);
    sda_oen <= 1'b0;
    sda_out <= 1'b0;
    @(negedge scl_rising_edge);
    sda_oen <= 1'b1;
    //7E + R ACK
    wait(cnt == 28);
    @(negedge scl_falling_edge);
    sda_oen <= 1'b0;
    sda_out <= 1'b0;
    //slave data (64 bit)
    repeat(32)begin
    @(negedge scl_falling_edge);
    sda_oen <= 1'b0;
    sda_out <= 1'b0;
    @(negedge scl_falling_edge);
    sda_oen <= 1'b1;
    end
    //slave da ack
    wait(cnt == 101);
    @(negedge scl_falling_edge);
    sda_oen <= 1'b0;
    sda_out <= 1'b0;
    @(negedge scl_falling_edge);
    sda_oen <= 1'b1;
    //slave addr + w ack
    wait(cnt == 121);
    @(negedge scl_falling_edge);
    sda_oen <= 1'b0;
    sda_out <= 1'b0;
    @(negedge scl_rising_edge);
    sda_oen <= 1'b1;
    //slave addr + w ack
    wait(cnt == 149);
    @(negedge scl_falling_edge);
    sda_oen <= 1'b0;
    sda_out <= 1'b0;
    @(negedge scl_rising_edge);
    sda_oen <= 1'b1;
    //slave addr + r ack
    wait(cnt == 168);
    @(negedge scl_falling_edge);
    sda_oen <= 1'b0;
    sda_out <= 1'b0;
    //slave data
    wait(cnt == 169);
    @(negedge scl_falling_edge);
    sda_oen <= 1'b0;
    sda_out <= 1'b0;
    @(negedge scl_falling_edge);
    sda_oen <= 1'b0;
    sda_out <= 1'b1;
    @(negedge scl_falling_edge);
    sda_oen <= 1'b0;
    sda_out <= 1'b0;
    @(negedge scl_falling_edge);
    sda_oen <= 1'b0;
    sda_out <= 1'b1;
    @(negedge scl_falling_edge);
    sda_oen <= 1'b0;
    sda_out <= 1'b0;
    @(negedge scl_falling_edge);
    sda_oen <= 1'b0;
    sda_out <= 1'b1;
    @(negedge scl_falling_edge);
    sda_oen <= 1'b0;
    sda_out <= 1'b1;
    @(negedge scl_falling_edge);
    sda_oen <= 1'b0;
    sda_out <= 1'b0;
    @(negedge scl_falling_edge);
    sda_oen <= 1'b1;
    //7E+W ACK
    wait(cnt == 187);
    @(negedge scl_falling_edge);
    sda_oen <= 1'b0;
    sda_out <= 1'b0;
    @(negedge scl_rising_edge);
    sda_oen <= 1'b1;
    //DDR WRITE COMMAND ACK
     wait(cnt == 208);
    @(negedge scl_rising_edge);
    sda_oen <= 1'b0;
    sda_out <= 1'b0;
    @(negedge scl_falling_edge);
    sda_oen <= 1'b1;
    //DDR READ COMMAND ACK
    wait(cnt == 235);
    @(negedge scl_rising_edge);
    sda_oen <= 1'b0;
    sda_out <= 1'b0;
    @(negedge scl_falling_edge);
    sda_oen <= 1'b0;
    sda_out <= 1'b1;
    //DDR READ PAR BIT
    wait(cnt == 243);
    @(negedge scl_falling_edge);
    sda_oen <= 1'b0;
    sda_out <= 1'b0;
    @(negedge scl_rising_edge);
    sda_oen <= 1'b0;
    sda_out <= 1'b1;
    @(negedge scl_falling_edge);
    sda_oen <= 1'b1;
end

always@(posedge scl_line)begin
    cnt <= cnt + 1;
end

always@(posedge sclk)begin
    scl_line_reg_d1 <= scl_line;
    scl_line_reg_d2 <= scl_line_reg_d1;
    scl_line_reg_d3 <= scl_line_reg_d2;
end


endmodule
